
% Solves parts F and G

% April 26, 1999
% Developed by Ali H. Sayed

close all
clear all
clear;
clf;

N=20000;          % number of symbols to be generated

s=(sign(randn(1,N))+i*sign(randn(1,N)))/(sqrt(2));
                              % generates QPSK data
r=filter([1 0.5],1,s);        % Filters the data through the channel

iter=0;

display('PLEASE WAIT FOR APPROXIMATELY 200 ITERATIONS')

for var_v=0.004:0.001:0.2;            % variance of noise; 
 iter=iter+1


  y=r+sqrt(var_v/2)*randn(1,N)+sqrt(var_v/2)*i*randn(1,N); 
         % adds complex noise


 % We form the covariance matrix R_y
 % Its (1,1) entry is kept in terms of the noise
 % variance for generality

  Ry=[((5/4)+var_v) 1/2   0

      1/2  ((5/4)+var_v)  1/2

      0   1/2    ((5/4)+var_v)];


  Ryinv=inv(Ry);

  Delta = 1; 
  Rxy = [0.5 1 0];
  Ko = Rxy*Ryinv;        % Optimal equalizer

  s_hat=filter(Ko,1,y);  % Filters y(i) through equalizer
                         % to generate hat{s}(i-Delta)

  % Number of errors


    num_error(iter)=0;
    s_nl=zeros(1,N);
    sq2=sqrt(2)/2;    

    for j=1:N-Delta

       a=sign(real(s_hat(j+Delta)));
       if a == 0  
         a=1;
       end

       b=sign(imag(s_hat(j+Delta)));
       if b == 0  
         b=1;
       end

      s_nl(j)=sq2*(a+i*b);  %Output of decision device

      if abs(s_nl(j)-s(j)) > 0.1
         num_error(iter) = num_error(iter)+1;
      end
    end



    % Number of errors without equalization

    num_error_2(iter)=0;
    s_nl=zeros(1,N);

    for j=1:N

       a=sign(real(y(j)));
       if a == 0  
         a=1;
       end

       b=sign(imag(y(j)));
       if b == 0  
         b=1;
       end

      s_nl(j)=sq2*(a+i*b);  

      if abs(s_nl(j)-s(j)) > 0.1
         num_error_2(iter) = num_error_2(iter)+1;
      end
    end

end


var_v=0.004:0.001:0.2;
SNR=10*log10(1.25./var_v);
SER=num_error/N;
SER2=num_error_2/N;

subplot(221)
semilogy(SNR,SER,SNR,SER2,'-.');
grid;
title('SER');
xlabel('SNR (dB)');
ylabel('SER');
legend('with equalization','without equalization');
axis tight;

subplot(222)
plot(SNR,SER,SNR,SER2,'-.');
grid;
title('SER');
xlabel('SNR (dB)');
ylabel('SER');
legend('with equalization','without equalization');
axis tight;

