
% Solves part B

% May 3, 1999
% Developed by Ali H. Sayed

close all
clear all;


M=4;           % Number of antennas
theta=pi/6;    % Direction of arrival
theta2=pi/4;   % Second direction of arrival

d_over_lambda= 1/5;  %ratio d/lambda
var_v=0.2;           % variance of white noise

p=zeros(M,1);
p2=zeros(M,1);

  % We now generate the vectors p

for j=1:M
 p(j)=exp((j-1)*i*2*pi*d_over_lambda*cos(theta));
 p2(j)=exp((j-1)*i*2*pi*d_over_lambda*cos(theta2));
end


N=1000;      % Number of data points
T_s= 0.01;   % Sampling period to create discrete
             % sinusoid from continuous sinusoid

A=1.0;       % Amplitude of one sinusoid
A2=0.2;      % Amplitude of second sinusoid

s1=A*cos(2*pi*[1:N]*T_s);  %One sinusoid
s2=A2*sin(4*pi*[1:N]*T_s); %Second sinusoid
s=s1+s2;

v=sqrt(var_v/2)*randn(M,N)+i*sqrt(var_v/2)*randn(M,N); 
     % M complex noise vectors:
     % one for each antenna

R_v = var_v*eye(M,M);
IR_v= inv(R_v);

mmse = 1/(p'*IR_v*p);
Ko = p'*IR_v*mmse;      % Optimal gain vector


y=zeros(M,N);           % Vector of measurements
s_hat=zeros(1,N);       % Estimated signal
norm_imag=zeros(1,N);   % Norm of imaginary part of estimated output

for j=1:N
 y(:,j)  = p*s1(j)+p2*s2(j)+v(:,j);
 s_hat(j)= Ko*y(:,j);
 norm_imag(j)=abs(imag(s_hat(j)));
end


mmse_hat=(norm(s_hat-s1)^2)/N;

s_hat = real(s_hat);   % Keep real part



%%%% We now plot part of the incident wave, its estimate, 
%%%% and the real part of signals received at antennas.


Nplot=200;

figure
subplot(611)
plot(Nplot:3*Nplot,s1(Nplot:3*Nplot));
legend('Transmitted at 30o');

subplot(612)
plot(Nplot:3*Nplot,s2(Nplot:3*Nplot));
legend('Transmitted at 45o');

subplot(613)
plot(Nplot:3*Nplot,real(y(1,Nplot:3*Nplot)));
legend('At antenna 1');


subplot(614)
plot(Nplot:3*Nplot,real(y(2,Nplot:3*Nplot)));
legend('At antenna 2');


subplot(615)
plot(Nplot:3*Nplot,real(y(3,Nplot:3*Nplot)));
legend('At antenna 3');


subplot(616)
plot(Nplot:3*Nplot,real(y(4,Nplot:3*Nplot)));
legend('At antenna 4');


figure
subplot(411)
plot(Nplot:3*Nplot,s1(Nplot:3*Nplot),Nplot:3*Nplot,real(s_hat(Nplot:3*Nplot)),'r.');
legend('Transmitted at 30o','Estimated');

subplot(412)
plot(Nplot:3*Nplot,s1(Nplot:3*Nplot));
legend('Transmitted at 30o');

subplot(413)
plot(Nplot:3*Nplot,s2(Nplot:3*Nplot));
legend('Transmitted at 45o');

subplot(414)
plot(Nplot:3*Nplot,s(Nplot:3*Nplot));
legend('Incident wave');

real(mmse)
mmse_hat

