
% Solves part C

% May 3, 1999
% Developed by Ali H. Sayed

close all
clear all;


M=4;           % Number of antennas
theta=pi/6;    % Direction of arrival
theta2=pi/4;   % Second direction of arrival


d_over_lambda= 1/5;  %ratio d/lambda

%% Spatial correlation matrix M x M

R_v=zeros(M,M);
rho=0.85;

for j=1:M
 for k=1:M
  R_v(j,k)=0.85^(abs(j-k));
 end
end

IR_v= inv(R_v);

L=chol(R_v)';   % Cholesky factorization

p=zeros(M,1);
p2=zeros(M,1);
  % We now generate the vectors p

for j=1:M
 p(j)=exp((j-1)*i*2*pi*d_over_lambda*cos(theta));
 p2(j)=exp((j-1)*i*2*pi*d_over_lambda*cos(theta2));
end


N=1000;      % Number of data points
T_s= 0.01;   % Sampling period to create discrete
             % sinusoid from continuous sinusoid

A=1.0;       % Amplitude of sinusoid
A2=0.5;      % Amplitude of second sinusoid


s=zeros(1,N);
s1=A*cos(2*pi*[1:N]*T_s);  %One sinusoid
s=s1;

sq2=sqrt(2)/2;

e=sq2*randn(M,N)+i*sq2*randn(M,N);
v=L*e;                       % M complex noise vectors:
                             % one for each antenna


mmse = 1/(p'*IR_v*p);
Ko = p'*IR_v*mmse;      % Optimal gain vector


y=zeros(M,N);           % Vector of measurements
s_hat=zeros(1,N);       % Estimated signal
norm_imag=zeros(1,N);   % Norm of imaginary part of estimated output

for j=1:N
 y(:,j)  = p*s(j)+v(:,j);
 s_hat(j)= Ko*y(:,j);
 norm_imag(j)=abs(imag(s_hat(j)));
end


mmse_hat=(norm(s_hat-s)^2)/N;

s_hat = real(s_hat);   % Keep real part

%%% Let's now add another sinusoid


s2=A2*sin(4*pi*[1:N]*T_s); %Second sinusoid
s=s1+s2;

y2=zeros(M,N);           % Vector of measurements
s_hat2=zeros(1,N);       % Estimated signal


for j=1:N
 y2(:,j)  = p*s1(j)+p2*s2(j)+v(:,j);
 s_hat2(j)= Ko*y2(:,j);
end


mmse_hat2=(norm(s_hat2-s)^2)/N;

s_hat2 = real(s_hat2);   % Keep real part





%%%% We now plot part of the incident wave, its estimate, 
%%%% and the real part of signals received at antennas.


Nplot=200;

figure
subplot(511)
plot(Nplot:3*Nplot,s1(Nplot:3*Nplot));
legend('Transmitted at 30o');

subplot(512)
plot(Nplot:3*Nplot,real(y(1,Nplot:3*Nplot)));
legend('At antenna 1');


subplot(513)
plot(Nplot:3*Nplot,real(y(2,Nplot:3*Nplot)));
legend('At antenna 2');


subplot(514)
plot(Nplot:3*Nplot,real(y(3,Nplot:3*Nplot)));
legend('At antenna 3');


subplot(515)
plot(Nplot:3*Nplot,real(y(4,Nplot:3*Nplot)));
legend('At antenna 4');


figure
subplot(211)
plot(Nplot:3*Nplot,s1(Nplot:3*Nplot),Nplot:3*Nplot,real(s_hat(Nplot:3*Nplot)),'r.');
legend('Transmitted','Estimated');

mmse
mmse_hat



%%%% We now plot part of the incident wave, its estimate, 
%%%% and the real part of signals received at antennas.


Nplot=200;

figure
subplot(611)
plot(Nplot:3*Nplot,s1(Nplot:3*Nplot));
legend('Transmitted at 30o');

subplot(612)
plot(Nplot:3*Nplot,s2(Nplot:3*Nplot));
legend('Transmitted at 45o');

subplot(613)
plot(Nplot:3*Nplot,real(y(1,Nplot:3*Nplot)));
legend('At antenna 1');


subplot(614)
plot(Nplot:3*Nplot,real(y(2,Nplot:3*Nplot)));
legend('At antenna 2');


subplot(615)
plot(Nplot:3*Nplot,real(y(3,Nplot:3*Nplot)));
legend('At antenna 3');


subplot(616)
plot(Nplot:3*Nplot,real(y(4,Nplot:3*Nplot)));
legend('At antenna 4');


figure
subplot(411)
plot(Nplot:3*Nplot,s1(Nplot:3*Nplot),Nplot:3*Nplot,real(s_hat2(Nplot:3*Nplot)),'r');
legend('Transmitted at 30o','Estimated');

subplot(412)
plot(Nplot:3*Nplot,s(Nplot:3*Nplot));
legend('Incident wave');


mmse
mmse_hat2