
% Solves part H

% April 29, 1999
% Developed by Ali H. Sayed



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Remark: This code uses the following notation
% N+1  is number of feedforward taps  (text uses L)
% N    is order  of feedforward filter (text uses L-1)
% Q    is number of feedback taps (text uses Q)
% M+1  is number of channel taps (text uses M)
% M    is order of channel (text uses M+1)
%
% In other words, the text works with the number of taps and 
% denotes them by (L,Q,M). The program works with filter
% orders instead and denotes them by (N,Q,M).
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


close all;
clear all;


c=[0.5 1.2 1.5 -1];  % Channel coefficients: LENGTH CAN NOT CHANGE

var_v=0.078;        % variance of noise
var_s=1;          % variance of input data
L=2000;           % number of symbols to be generated

M=length(c)-1;    % length of channel
N_max=20;         % maximum length of feedforward filter
Delta=4;          % Delay: CAN NOT CHANGE


s=(sign(randn(1,L))+i*sign(randn(1,L)))/(sqrt(2));
                              % generates QPSK data

r=filter(c,1,s);              % Filters the data through the channel


y=r+sqrt(var_v/2)*randn(1,L)+sqrt(var_v/2)*i*randn(1,L); 
                              % adds complex noise to it


display('PLEASE WAIT FOR FEW ITERATIONS')


for N=1:N_max    % Changing the length of the linear equalizer

  N

  % We form the covariance matrix R_y and its inverse

   CT=zeros(N+1,N+M+1);
   R_y=zeros(N+1,N+1);
   IR_y=zeros(N+1,N+1);

   col=zeros(N+1,1);
   col(1)=c(1);
   row=zeros(1,N+M+1);
   row(1:M+1)=c;

   CT = toeplitz(col,row); %  prewindowed Toeplitz channel matrix

   R_under_s=var_s*eye(N+M+1,N+M+1);

   R_v=var_v*eye(N+1,N+1);
   R_y=CT*R_under_s*CT'+R_v;
   IR_y=inv(R_y);

   Rxy=zeros(1,N+1);


   av=var_s*c(4);
   bv=var_s*c(3);
   cv=var_s*c(2);
   dv=var_s*c(1);


   switch N

   case 1, Rxy = [0 av];

   case 2, Rxy = [0 av bv];

   case 3, Rxy = [0 av bv cv];

   case 4, Rxy = [0 av bv cv dv];

   otherwise, Rxy(2:5)=[av bv cv dv];

   end




   
   Ko=zeros(1,N+1);
   Ko = Rxy*IR_y;       % Optimal equalizer

   s_hat=zeros(1,L+N);
   s_check=zeros(1,L+N);

   s_hat=filter(Ko,1,y); % Filter y(i) through equalizer
                         % to generate hat{s}(i-Delta)

   for j=1:L
    s_check(j)=dec(s_hat(j));   %Output of decision device
   end


   mmse(N)=var_s-Rxy*IR_y*Rxy';  % Theoretical mmse

   % Estimate of m.m.s.e.

    mmse_hat(N)=0;
 
    for j=1:L-Delta
      mmse_hat(N)=mmse_hat(N)+(s_hat(j+Delta)-s(j))*(s_hat(j+Delta)-s(j))';
    end
 
    mmse_hat(N)=mmse_hat(N)/(L-Delta);



   % Number of errors

     num_error(N)=0;
 
     for j=1:L-Delta
       if abs(s_check(j+Delta)-s(j)) > 0.1
         num_error(N) = num_error(N)+1;
       end

     end

    if N == 4
      input_device=s_hat;  %%% Save the input to the nonlinear
                           %%% device for N=4.
    end

end


%num_error


%%%% We now plot all symbols in constellation 
%%%% form for both the received signal and 
%%%% the signal at the input of the decision
%%%% device

figure
subplot(221);
plot(y,'.');grid;
title('Received sequence');

subplot(222);
plot(input_device,'.');grid;
title('Input of decision device');



subplot(223)
stem(1:N_max,num_error);
grid;
xlabel('Length of filter');
ylabel('errors');


subplot(224)
plot(1:N_max,mmse,1:N_max,mmse_hat,'-.');
grid;
xlabel('Length of filter');
ylabel('mmse');
legend('Theoretical mmse','Estimated mmse');
axis tight;
