
% Solves part I

% April 29, 1999
% Developed by Ali H. Sayed



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Remark: This code uses the following notation
% N+1  is number of feedforward taps  (text uses L)
% N    is order  of feedforward filter (text uses L-1)
% Q    is number of feedback taps (text uses Q)
% M+1  is number of channel taps (text uses M)
% M    is order of channel (text uses M+1)
%
% In other words, the text works with the number of taps and 
% denotes them by (L,Q,M). The program works with filter
% orders instead and denotes them by (N,Q,M).
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


close all
clear all


c=[0.5 1.2 1.5 -1];  % Channel coefficients

var_s=1;          % variance of input data
var_v=0.078;        % variance of noise
L=2000;           % number of symbols to be generated

M=length(c)-1;  % length of channel
N=6;            % length of feedforward filter is N+1
Q=1;            % length of feedback filter
Delta=5;        % Delay

NT=200;         % Number of symbols used for training

s=(sign(randn(1,L))+i*sign(randn(1,L)))/(sqrt(2));
                              % generates QPSK data

r=filter(c,1,s);              % Filters the data through the channel

y=r+sqrt(var_v/2)*randn(1,L)+sqrt(var_v/2)*i*randn(1,L); 
                              % adds complex noise to it


%%%%% We first form the data matrix H


col=s(1:NT)';
row=zeros(1,M+1);
row(1)=s(1);

H = toeplitz(col,row); 

c_hat2=(inv(H'*H))*H'*y(1:NT)';
c_hat=real(c_hat2)';   % estimated channel impulse response in row form
                       % We are only keeping the real part.

var_v_hat = (norm(y(1:NT)'-H*c_hat2))^2/(NT);



f1=figure(1);
subplot(221), stem(c); grid;
hold
stem(c_hat,'filled');
title('Channel impulse response');
xlabel('i');
legend('Actual','Estimated')
subplot(222); 
[Hc,Wc]=freqz(c,1,512,'whole');
[Hh,Wh]=freqz(c_hat,1,512,'whole');
xf=0:2*pi/512:2*pi; % Frequency axis is from 0 to 2 pi
plot(xf(1:length(xf)-1),20*log10(abs(Hc)),xf(1:length(xf)-1),20*log10(abs(Hh)));
title('Frequency response'); grid;
xlabel('\omega');
ylabel('dB');
legend('Actual','Estimated');



var_v_hat



%%%%%%%%%%% Design of equalizer


col=zeros(N+1,1);
col(1)=c_hat(1);

row=zeros(1,N+M+1);
row(1:M+1)=c_hat;

CT = toeplitz(col,row); %  prewindowed Toeplitz channel matrix

R_v=var_v_hat*eye(N+1,N+1);
R_under_s=var_s*eye(N+M+1,N+M+1);
R_s=var_s*eye(Q+1,Q+1);

R_y=CT*R_under_s*CT'+R_v;
IR_y=inv(R_y);

% Calculate Rsy

if Delta <= N+M-Q
  XT=CT';
  Rsy=var_s*XT(1+Delta:Delta+Q+1,1:N+1);
else
  XT=CT';
  Rsy(1:N+M-Delta+1,:)=var_s*XT(1+Delta:N+M+1,1:N+1);
  Rsy(N+M-Delta+2:Q+1,:)=zeros(Delta-N-M+Q,N+1);
end

 % Calculate R_delta and its inverse

R_delta = R_s-Rsy*IR_y*Rsy';
IR_delta = inv(R_delta);


% Get the optimal coefficients for the DFE

bopt=IR_delta(1,:)/IR_delta(1,1);  
fopt=bopt*Rsy*IR_y;
    
beq=bopt(2:Q+1);    

% Calculate s_hat and s_check

s_check=zeros(1,L+Q+1);
s_hat=zeros(1,L+Q+1);
y_1=zeros(1,L+Q+1);

y_1=filter(fopt,1,y);   % Output of the feedforward filter


% Let us compute the first Q outputs \hat{s} directly


s_hat(1)=y_1(1);
s_check(1)=dec(s_hat(1));  % Since initial conditions in B(z)
                           % are zero.


for j=2:Q+1
 s_hat(j)=y_1(j)-beq(1:j-1)*s_check(j-1:-1:1).';
 s_check(j)=dec(s_hat(j));
end

%% Now we compute the remaining equalizer outputs

for j=Q+2:L
  s_hat(j)=y_1(j)-beq*s_check(j-1:-1:j-Q).';
  s_check(j)=dec(s_hat(j));
end


%% Number of errors    

  num_error=0;

  for j=1:L-Delta
    if abs(s_check(j+Delta)-s(j)) > 0.1
      num_error = num_error+1;
    end
  end


% Estimate of m.m.s.e.

    mmse_hat=0;
 
    for j=1:L-Delta
     mmse_hat=mmse_hat+(s_hat(j+Delta)-s(j))*(s_hat(j+Delta)-s(j))';
    end
 
    mmse_hat=mmse_hat/(L-Delta);

    input_device=s_hat;  


%%%% We now plot all symbols in constellation 
%%%% form for both the received signal and 
%%%% the signal at the input of the decision
%%%% device

subplot(223);
plot(y,'.');grid;
title('Received sequence');

subplot(224);
plot(input_device,'.');grid;
title('Input of decision device');

num_error

mmse_hat

c_hat