%
% This program solves part (a). It allows the user
% to change the value of gamma, the initial condition
% w_init, the duration of the simulation N, and
% the number of experiments, L.
%

% Written by Ali H. Sayed
% Jan. 19, 2000

clear all;
close all;

gamma=1;

Ru = [1 0
      0 2];

N=500;                % length of simulation
mu=0.001;             % step-size

J=zeros(1,N);         % Learning curve
w_init=[-0.8 0.8]';   % initial condition
w=w_init;
J(1)=gamma^2+2*(w(1)^2+2*w(2)^2)^2-2*gamma*(w(1)^2+2*w(2)^2);

for i=2:N
 w=w+mu*(gamma-2*w(1)^2-4*w(2)^2)*[w(1) 2*w(2)]';
 J(i)=gamma^2+2*(w(1)^2+2*w(2)^2)^2-2*gamma*(w(1)^2+2*w(2)^2);
end

% Ensemble-average learning curve

L=500;  % Number of experiments
J_hat=zeros(1,N);
J_avg=zeros(1,N);

sq2=1/sqrt(2);

display('Wait for 200 iterations');

for l = 1:L
  l
  w_init=[-0.8 0.8]';
  
  ua=sq2*randn(N,1);   % Generation of complex circular variables
  ub=sq2*randn(N,1);
  uc=randn(N,1);
  ud=randn(N,1);

  u1=ua + j*ub;
  u2=uc + j*ud;

  w=w_init;
  
  for i=1:N
   u=[u1(i) u2(i)];
   z=u*w;
   J_hat(i)=(gamma-abs(z)^2)^2;
   w=w+mu*u'*z*(gamma-abs(z)^2);
  end;

 J_avg = J_avg + J_hat;

end;

J_avg = J_avg/L;

figure
plot(0:N-1,J,'linewidth',2, 'color','r');
hold on
plot(J_avg);
hold off
title('Learning curves for steepest-descent and CMA2-2');
xlabel('Number of iteration');
ylabel('J(i)');
%axis([1 500 0 6]);
legend('Steepest-descent', 'CMA2-2');
grid
