
%
% This program solves part (b). It allows the user
% to change the value of gamma, the initial condition
% w_init, and the duration of the simulation N.


% Written by Ali H. Sayed
% Jan. 19, 2000

clear all;
close all;

gamma=1;

Ru = [1 0
      0 2];



w1o= [-1:0.01:1];  % range of values for grid
w2o= [-1:0.01:1]';
L = length(w1o);

J=zeros(L,L);
for i=1:L
 for k=1:L
  x=w1o(i)^2;
  y=w2o(k)^2;
  c=x+2*y;
  J(k,i)=gamma*gamma + 2*c*c - 2*gamma*c;
 end
end    


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

N=1000;
we=zeros(2,N);
wsd=zeros(2,N);
w_init = [-0.8 0.8]';   % First initial condition

w=w_init;
ws=w_init;
we(:,1)=w_init;
wsd(:,1)=w_init;
mu = 0.001;


sq2=1/sqrt(2);

ua=sq2*randn(N,1);   % Generation of complex circular variables
ub=sq2*randn(N,1);
uc=randn(N,1);
ud=randn(N,1);

j=sqrt(-1);
u1=ua + j*ub;
u2=uc + j*ud;


for i = 1:N
   u=[u1(i) u2(i)];
   z=u*w;
   w=w+mu*u'*z*(gamma-abs(z)^2);  %CMA2-2

   we(:,i)=real(w);  
   ws=ws+mu*(gamma-2*ws(1)^2-4*ws(2)^2)*[ws(1) 2*ws(2)]'; %Steepest-descent
   wsd(:,i)=ws;  
end;




%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

N=1000;
we2=zeros(2,N);
wsd2=zeros(2,N);
w_init = [0.8 -0.8]';   % Second initial condition

w=w_init;
ws=w_init;
we2(:,1)=w_init;
wsd2(:,1)=w_init;
mu = 0.001;

for i = 1:N
   u=[u1(i) u2(i)];
   z=u*w;
   w=w+mu*u'*z*(gamma-abs(z)^2);  %CMA2-2

   we2(:,i)=real(w);  
   ws=ws+mu*(gamma-2*ws(1)^2-4*ws(2)^2)*[ws(1) 2*ws(2)]'; %Steepest-descent
   wsd2(:,i)=ws;  
end;



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

N=1000;
we3=zeros(2,N);
wsd3=zeros(2,N);
w_init = [-0.8 -0.8]';   % Third initial condition

w=w_init;
ws=w_init;
we3(:,1)=w_init;
wsd3(:,1)=w_init;
mu = 0.001;

for i = 1:N
   u=[u1(i) u2(i)];
   z=u*w;
   w=w+mu*u'*z*(gamma-abs(z)^2);  %CMA2-2

   we3(:,i)=real(w);  
   ws=ws+mu*(gamma-2*ws(1)^2-4*ws(2)^2)*[ws(1) 2*ws(2)]'; %Steepest-descent
   wsd3(:,i)=ws;  
end;


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


N=1000;
we4=zeros(2,N);
wsd4=zeros(2,N);
w_init = [0.8 0.8]';   % Fourth initial condition

w=w_init;
ws=w_init;
we4(:,1)=w_init;
wsd4(:,1)=w_init;
mu = 0.001;

for i = 1:N
   u=[u1(i) u2(i)];
   z=u*w;
   w=w+mu*u'*z*(gamma-abs(z)^2);  %CMA2-2

   we4(:,i)=real(w);  
   ws=ws+mu*(gamma-2*ws(1)^2-4*ws(2)^2)*[ws(1) 2*ws(2)]'; %Steepest-descent
   wsd4(:,i)=ws;  
end;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

disp('Final values of weight estimates');
disp(['w_{-1}=[-0.8  0.8] ==> final weight estimate = ', num2str(we(:,N)')]);
disp(['w_{-1}=[ 0.8 -0.8] ==> final weight estimate = ', num2str(we2(:,N)')]);
disp(['w_{-1}=[-0.8 -0.8] ==> final weight estimate = ', num2str(we3(:,N)')]);
disp(['w_{-1}=[ 0.8  0.8] ==> final weight estimate = ', num2str(we4(:,N)')]);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

min_J=min(min(J));   % minimum cost
max_J=max(max(J));   
levels=[min_J+0.001:0.05:1,1,1:2:max_J];
contour(w1o, w2o,J,levels);
colorbar;
xlabel('w_i(1)');
ylabel('w_i(2)');
s = strcat('Contour curves and trajectories for CMA2-2');
title(s);
hold on
plot(we(1,:),  we(2,:),'xm--');
plot(we2(1,:), we2(2,:),'xm--');

plot(we3(1,:), we3(2,:),'xm--');
plot(we4(1,:), we4(2,:),'xm--');
hold off
zoom on

figure
contour(w1o, w2o,J,levels);
colorbar;
xlabel('w_i(1)');
ylabel('w_i(2)');
s = strcat('Contour curves and trajectories for steepest-descent');
title(s);
hold on
plot(wsd(1,:),  wsd(2,:),'xr--');
plot(wsd2(1,:), wsd2(2,:),'xr--');
plot(wsd3(1,:), wsd3(2,:),'xr--');
plot(wsd4(1,:), wsd4(2,:),'xr--');
hold off
zoom on

