%Adaptive equalization project from chapter 5
%Solves part B
%Developed by A. H. Sayed

clear all;  
close all

qam = 256 % Choose from 4/16/64/256
SNR = 30 % SNR at output of channel
equalizer_length = 35;
Delta =15; %approx. delay by channel + half length of equalizer
mu_nlms = 0.4;
mu_lms=0.001;
epsilon = 1e-6; 

ltrain=[150 300 500];
disp('This takes some time...')

for alg=1:2
  % alg=1 --> LMS
  % alg=2 --> NLMS
 if alg==1
     disp('LMS')
 else
     disp('NLMS')
 end
 
 for kk=1:3
 clear train_data s hat_s check_s r d y e w v training N u type sigma_v_dd sigma_v_tt sigma_s x xint xcoor
 
 if kk == 1
     disp('150 training symbols');
 else if kk==2
         disp('300 training symbols');
     else 
         disp('500 training symbols');
     end
 end
 training = ltrain(kk);  % length of training with BPSK data
 decision_directed = 5000; % length of decision-directed with QAM data
 N = training + decision_directed; % total number of symbols

 train_data = zeros(1,training+Delta); %training data
 s=zeros(1,N);
 hat_s=zeros(1,N);
 check_s=zeros(1,N);
 r=zeros(1,N);
 d=zeros(1,N);
 y=zeros(1,N);

%Channel
 channel=[0.5 1.2 1.5 -1]; % from chapter 3. 

%Noise variance during decision-directed
%QAM symbols don't have unit-variance
switch qam
case 4
   sigma_s = sqrt(2);
case 16
   sigma_s = sqrt(10);
case 64
   sigma_s = sqrt(42);
case 256
   sigma_s = sqrt(170);
end
sigma_v_dd = sqrt((sigma_s)^2*norm(channel)^2/10^(SNR/10)); % during decision-directed
sigma_v_tt = sqrt(norm(channel)^2/10^(SNR/10)); % during training

v=zeros(1,N);
v(1:training)=(sigma_v_tt/2)*(randn(1,training)+j*randn(1,training));
v(training:N)=(sigma_v_dd/2)*(randn(1,N-training+1)+j*randn(1,N-training+1));


%Data
s(1:training)=(sign(randn(1,training))+j*sign(randn(1,training)))/(sqrt(2)); 
train_data(Delta+1:training+Delta)=s(1:training); % QPSK training data
   % generates QPSK data; it has unit variance
   % The training vector starts with Delta zeros.
   
for i=1:decision_directed
 x=rand;   
 xint=round((qam-1)*x); % generates integers, e.g., between 0 and 255
 xcoor=modmap(xint,1,1,'qam',qam); % gives real and imaginary coordinates
 s(training+i)=xcoor(1) + j*xcoor(2); % combines them into a complex number
end 
 
y=filter(channel,1,s); % filters data through channel
r=y+v; 

% Equalizer initialization
w  = zeros(equalizer_length,1); % equalizer coefficients (column) 
u  = zeros(1,equalizer_length); % regressor vector (row) 
e = zeros(1,N); % error vector
num_errors=0;

%  Adaptive Equalization

% Training mode
for i = 1:training+Delta
   u  = [r(i) u(1:equalizer_length-1)];
   hat_s(i) = u*w;
   d(i) = train_data(i); % training
   e(i) = d(i)- hat_s(i);
   if alg == 2
     w = w + mu_nlms*e(i)*u'/(norm(u)^2+epsilon);
   else
     w= w + mu_lms*e(i)*u';
   end
end


% Decision-directed mode
for i = training+Delta+1:N
   u  = [r(i) u(1:equalizer_length-1)];
   hat_s(i) = u*w;
   switch qam 
    case 4
      check_s(i)=slicer4(hat_s(i));
    case 16
      check_s(i)=slicer16(hat_s(i));
    case 64
      check_s(i)=slicer64(hat_s(i));
    case 256
      check_s(i)=slicer256(hat_s(i));
   end
   
   d(i) = check_s(i); % decision_directed
   e(i) = d(i)- hat_s(i);
   if alg == 2
     w = w + mu_nlms*e(i)*u'/(norm(u)^2+epsilon);
   else
     w = w + mu_lms*e(i)*u';
   end

   if (check_s(i) ~= s(i-Delta))
     num_errors = num_errors + 1;
   end
   
end  

if alg == 1
 switch kk
  case 1
   subplot(331); 
   plot(real(hat_s(training+Delta+1:N)),imag(hat_s(training+Delta+1:N)),'.');grid; 
   title('150/LMS');
  case 2
   subplot(332); 
   plot(real(hat_s(training+Delta+1:N)),imag(hat_s(training+Delta+1:N)),'.');grid; 
   title('300/LMS');
  case 3
   subplot(333); 
   plot(real(hat_s(training+Delta+1:N)),imag(hat_s(training+Delta+1:N)),'.');grid; 
   title('500/LMS');
 end
else
 switch kk
  case 1
   subplot(334); 
   plot(real(hat_s(training+Delta+1:N)),imag(hat_s(training+Delta+1:N)),'.');grid; 
   title('150/NLMS');
  case 2
   subplot(335); 
   plot(real(hat_s(training+Delta+1:N)),imag(hat_s(training+Delta+1:N)),'.');grid; 
   title('300/NLMS');
  case 3
   subplot(336); 
   plot(real(hat_s(training+Delta+1:N)),imag(hat_s(training+Delta+1:N)),'.');grid; 
   title('500/NLMS');
  end
end   
 num_errors
end
end 
 
 
 

