% Solves part G

close all
clear all

var_n = 1e-3;
sqn = sqrt(var_n);

Ts = 0.8e-6; % sampling frequency is 1.25MHz
Ns = 40000;
fd = 10; % Doppler frequencies

M = 5;
f = bessel(0,2*pi*fd*Ts); 
q = (1-(f*f)); 
Q = q*eye(M,M);
R = eye(M,M);

mu_opt = sqrt(trace(Q)/(var_n*trace(R))); % approx. 0.001
mu_vec = [1e-5 3e-5 5e-5 7e-5 1e-4 3e-4 5e-4 7e-4 1e-3 3e-3 5e-3 7e-3 1e-2 3e-2 5e-2 7e-2];
%mu_vec = [5e-3 7e-3 1e-2 3e-2 5e-2 7e-2];

K = length(mu_vec);

ray1 = rayleigh(Ts,Ns,fd);
ray2 = rayleigh(Ts,Ns,fd);
N = Ns;

for i=1:K
  mu = mu_vec(i);
  mu
  learn = zeros(1,N);
  L = 50;
  M = 5;
  R = eye(M,M);

  for k = 1:L
   input = randn(1,N);
   v = sqn*randn(1,N); 
   e = zeros(1,N);
   u = zeros(1,M);
   w = zeros(M,1);
   for j=1:N
     wo = [0 0 ray1(j) 0 ray2(j)]';
     u = [input(j) u(1:M-1)];
     d = u*wo + v(j);
     e(j) = d - u*w;
     w = w + mu*u'*e(j);
   end 
   learn = learn + abs(e).^2;
  end
  learn = learn/L;
  mse(i) = mean(learn(N-200:N))
  thy(i) = var_n + (mu*var_n*trace(R) + (1/mu)*trace(Q))/(2-mu*trace(R))
 end


 figure
 plot(mu_vec,10*log10(mse),'r-o',mu_vec,10*log10(thy),'g-d')
 ylabel('MSE (dB)')
 xlabel('\mu');
 legend('Simulation','Theory');
 axis tight
 grid

