
% Solves part B
% LMS
% A. H. Sayed

 close all;
 clear all;

 N = 10000;  % number of iterations
 L = 30;     % number of experiments

 M = 5; % number of taps
 wo = randn(M,1);
 wo = wo/norm(wo);

 var_n = 0.001; % Gaussian noise
 sqn   = sqrt(var_n);

 mu = 0.03;

 B = [9 10 11 12 13 14 15]; % word-lengths
 K = length(B); 

 for l=1:K
     
   Br = B(l)
   Bc = B(l);
   wo = quantize(wo,Bc);
   
   learn = zeros(1,N);
   for k=1:L
      input = randn(1,N+M); % input data
      v = sqn*randn(1,N+M); %  white Gaussian noise 
      w = zeros(M,1); 

      for i=1:N
         u = quantize(input(i:i+M-1),Br); 
         d = u*wo + v(i);
         d = quantize(d,Br);
         d_hat = u*w;
         d_hat = quantize(d_hat,Br);
         e(i) = quantize(d-d_hat,Br);
         w = w + quantize((mu*u'*e(i)),Bc);  % LMS algorithm
         w = quantize(w,Bc);
      end;
      learn = learn + abs(e).^2;
   end
                 
  learn = learn/L;
  
  var_c = 2^(-2*(Bc-1))/12;
  var_r = 2^(-2*(Br-1))/12;

  R = eye(M,M);
  var_n_bar = var_n +  3*var_r + 2*var_c*trace(R);

  Q = 0;
  Q_bar = Q + 2*var_c*eye(M,M);
  R_bar = R + var_c*eye(M,M);
  
  a = mu*var_n_bar*trace(R_bar);
  b = (1/mu)*trace(Q_bar);
  c = (2-mu*trace(R_bar));

  sim(l) = sum(learn(N-999:N))/1000
  thy(l) = var_n + ((a+b)/c) + 2*var_c*trace(R)
end

format long
[sim' thy' B']

figure
plot(B, 10*log10(sim),'ro-',B, 10*log10(thy),'bd-')
grid
xlabel('Bits (B_c = B_r)')
ylabel('MSE (dB)')
legend('Simulation','Theory')



