
% Solves part C
% Quantized e-NLMS
% A. H. Sayed

 close all;
 clear all;

 N = 30000;  % number of iterations
 L = 30;     % number of experiments

 Br = 13;    % number of data bits (including sign bit)
 Bc = 13;    % number of coefficient bits (including sign bit)
 
 M = 5; % number of taps
 wo = randn(M,1);
 wo = wo/norm(wo);
 wo = quantize_v(wo,Bc);

 var_n = 0.001; % Gaussian noise
 sqn   = sqrt(var_n);

 mu_vec = [0.01 0.02 0.05 0.08 0.1 0.2 0.3 0.5 0.7 1.0];
 K = length(mu_vec);

 epsilon = 1e-6;
 for l=1:K
   mu = mu_vec(l);
   learn = zeros(1,N);
   av2 = 0;
   av4 = 0;
   
   for k=1:L
       
      input = randn(1,N+M); % input data
      v = sqn*randn(1,N+M); %  white Gaussian noise 
      w = zeros(M,1); 

      k
      for i=1:N
         u = quantize_v(input(i:i+M-1),Br); 
         d = u*wo + v(i);
         d = quantize(d,Br);
         d_hat = u*w;
         d_hat = quantize(d_hat,Br);
         e(i) = quantize(d-d_hat,Br);
         aux = norm(u)^2;
         factor = mu/(epsilon + aux);
         factor = quantize(factor,Br);
         w = w + quantize_v((u'*factor*e(i)),Bc);  % e-NLMS algorithm
         w = quantize_v(w,Bc);
         av2 = av2 + (1/aux);  % estimates E(1/||u||^2)
         av4 = av4 + 1/(aux*aux); % estimates E(1/||u||^4)
      end;
      learn = learn + abs(e).^2;
   end
                 
  learn = learn/L;
  av2 = av2/(L*N);
  av4 = av4/(L*N);

  var_c = 2^(-2*(Bc-1))/12;
  var_r = 2^(-2*(Br-1))/12;

  var_n_bar = var_n +  3*var_r  + 2*var_c*trace(R);

  R = eye(M,M);
  Q = 0;
  Q_bar = Q + 2*var_c*eye(M,M) + mu*mu*var_r*(eye(M,M)+var_r*eye(M,M));
 
  a = mu*var_n_bar*(av2 + M*var_c*av4);
  b = (1/mu)*trace(Q_bar);
  c = (2-mu)*av2 - mu*M*var_c*av4;
  sim(l) = sum(learn(N-999:N))/1000
  thy(l) = var_n + ((a+b)/c) + 2*var_c*trace(R);
end

figure(1)
plot(mu_vec, 10*log10(sim),'or--',mu_vec, 10*log10(thy),'db-')
grid
xlabel('\mu')
ylabel('MSE')
legend('Simulation','Theory')



