
% Solves part D
% Quantized LMMN

 close all;
 clear all;

 N = 10000;  % number of iterations
 L = 30;     % number of experiments

 Br = 10;    % number of data bits (including sign bit)
 Bc = 10;    % number of coefficient bits (including sign bit)
 
 M = 5; % number of taps
 wo = randn(M,1);
 wo = wo/norm(wo);
 wo = quantize_v(wo,Bc);

 var_n = 0.001; % Gaussian noise
 sqn   = sqrt(var_n);
 
 var_un = 0.01*12;  % Uniform noise of variance 0.01 (the factor 12 is because uniform
                    % random variables between -1/2 and 1/2 have variance 1/12.
 squn  = sqrt(var_un);

 delta = 0.5;
 mu_vec = [0.01:0.01:0.1];
 K = length(mu_vec);

 for l=1:K
   mu = mu_vec(l);
   learn = zeros(1,N);
   v2_av = 0;
   v4_av = 0;
   v6_av = 0;

   for k=1:L
       
      input = randn(1,N+M); % input data
      v_g = sqn*randn(1,N+M); %  white Gaussian noise 
      v_u = squn*(rand(1,N+M)-0.5); % white uniform noise
      v = v_g + v_u;
      w = zeros(M,1); 

      v2_av = v2_av + mean(v.^2); % 2nd moment
      v4_av = v4_av + mean(v.^4); % 4th moment
      v6_av = v6_av + mean(v.^6); % 6th moment

      k
      for i=1:N
         u = quantize_v(input(i:i+M-1),Br); 
         d = u*wo + v(i);
         d = quantize(d,Br);
         d_hat = u*w;
         d_hat = quantize(d_hat,Br);
         e(i) = quantize(d-d_hat,Br);
         if abs(e(i))<1
            en = quantize((delta*e(i)),Br) + quantize(((1-delta)*(e(i))^3),Br);
            w = w + quantize_v((mu*u'*en),Bc);  % LMMN algorithm
            w = quantize_v(w,Bc);
         else
            w = w + quantize_v((mu*u'*e(i)),Bc);  % LMS algorithm
            w = quantize_v(w,Bc);
         end
      end;
      learn = learn + abs(e).^2;
  end

                 
 learn = learn/L;
 v2_av = v2_av/L;
 v4_av = v4_av/L;
 v6_av = v6_av/L;

 var_c = 2^(-2*(Bc-1))/12;
 var_r = 2^(-2*(Br-1))/12;

 n2 = var_n;
 u2 = var_un/12 + 3*var_r;

 u  = sqrt(3*u2);
 u4 = (u^4)/5;
 u6 = (u^6)/7;

 n4 = 3*n2^2;
 n6 = 15*n2^3;

 sv2 = n2 + u2;  % var_v_bar
 sv4 = n4 + u4 + 4*n2*u2;
 sv6 = n6 + 15*n4*u2 + 15*n2*u4 + u6;

 
 a = (delta^2*sv2 + 2*delta*(1-delta)*sv4 + (1-delta)^2*sv6);
 b = delta + 3*(1-delta)*sv2;
 c = delta^2 + 12*delta*(1-delta)*sv2 + 15*(1-delta)^2*sv4;

 R = eye(M,M);
 R_bar = R + var_c*R; 
 Q = 0;
 Q_bar = Q + 2*var_c*eye(M,M) + 2*mu*mu*var_r*(eye(M,M)+var_r*eye(M,M));
 
 sim(l) = sum(learn(N-999:N))/1000
 thy(l) = v2_av + ( (mu^(-1))*trace(Q_bar) + mu*a*trace(R_bar))/(2*b - mu*c*trace(R_bar)) + 2*var_c*trace(R);
end

figure
plot(mu_vec, 10*log10(sim),'vr--',mu_vec, 10*log10(thy),'xb-')
grid
xlabel('\mu')
ylabel('MSE (dB)')
legend('Simulation','Theory')
axis tight