

% Solves part E
% Transient performance of LMS using error nonlinearity results
% A. H. Sayed
% March 2002

 close all;
 clear all;

 N = 1000;  % number of iterations
 
 M = 5; % number of taps
 wo = randn(M,1);
 wo = wo/norm(wo);

 var_un = 0.001; % uniform noise with variance 0.001
 squn = sqrt(var_un*12);
 
 mu = 0.05;

 rho = 5;  % desired eigenvalue spread
 lambda_min = 1;   % generate positive eigenvalues in the interval
 lambda_max = rho; % [lambda_min; lambda_max];
 eigenv = lambda_min + (lambda_max-lambda_min)*rand(M-2,1);
 R = diag([lambda_min;eigenv;lambda_max]); % Covariance matrix; it is diagonal = Lambda
 sqR = sqrt(R);
 
 lambda = [lambda_min;eigenv;lambda_max]';
 p = poly(lambda); % coefficients of the characteristic polynomial (since R is diagonal)
 p = fliplr(p); 
 
 L = 50;

 w  = zeros(M,1); % initial weight estimate
 u  = zeros(1,M); % initial regressor

 msd_curve_sim = zeros(1,N); % simulated MSD curve     


 %%%%%%%%%%%%%%%%% GENERATE ENSEMBLE-AVERAGE MSD CURVE 

 for j=1:L % iteration over learning curves
   j 
   w = zeros(M,1);
   nw = zeros(1,N);
   
   for i=1:N  
     nw(i) = norm(wo-w)^2;
     u = sqrt(12)*(rand(1,M)-0.5)*sqR; % regressor/ uniform distribution with variance R
     d = u*wo + squn*(rand-0.5); % reference with uniform noise
     error = d - u*w; % error
     w = w + mu*u'*error;  % LMS recursion
   end
        
   msd_curve_sim = msd_curve_sim + nw;
end

msd_curve_sim  = msd_curve_sim/L;
msd_value  = mean(msd_curve_sim(N-200:N));


%%%%%%%%%%%%%%%%% GENERATE THEORETICAL MSD CURVE

 wtilde = wo; % initial condition
 W = zeros(M,1); % state vector
 Y = zeros(M,1);
 Rp1 = eye(M,M);
 Rp2 = R;
 for i=1:M
    Y(i) = trace(Rp2);
    W(i) = wtilde'*Rp1*wtilde;
    Rp2 = R*Rp2;
    Rp1 = R*Rp1;
 end
     
 hg = 1;  % h_g for LMS
 hG = real(hg);
 hU = var_un + W(2); % h_U for LMS

 F = zeros(M,M);
 F(1,1) = 1;
 F(1,2) = -2*mu*hG;
 
 for i=2:M-1
  F(i,:) = [0 F(i-1,1:M-1)];  
 end
 
 F(M,:) = 2*mu*hG*ones(1,M);
 F(M,:) = F(M,:).*p(1:M);
 F(M,M) = F(M,M)+1;

 for i=1:N
  msd(i) = W(1);
  W = F*W + mu*mu*hU*Y;
  hU = var_un + W(2);
 end


figure
plot(1:N,10*log10(msd_curve_sim),'r',1:N,10*log10(msd),'b-')
xlabel('i');
ylabel('MSD (dB)');
legend('Simulation','Theory');
axis tight
grid

