
% Solves part G
% Transient performance of LMF using error nonlinearity results
% A. H. Sayed
% March 2002

 close all;
 clear all;

 N = 100000;  % number of iterations
 
 M = 5; % number of taps
 wo = randn(M,1);
 wo = wo/norm(wo);

 var_n = 0.00001; % variance of Gaussian noise
 sqn = sqrt(var_n*12);
 xi_v_4 = 3*(var_n^2);  % fourth moment
 xi_v_6 = 15*(var_n^3); % sixth moment
 
 mu = 0.001;

 rho = 5;  % desired eigenvalue spread
 lambda_min = 1;   % generate positive eigenvalues in the interval
 lambda_max = rho; % [lambda_min; lambda_max];
 eigenv = lambda_min + (lambda_max-lambda_min)*rand(M-2,1);
 R = diag([lambda_min;eigenv;lambda_max]); % Covariance matrix; it is diagonal = Lambda
 sqR = sqrt(R);
 
 lambda = [lambda_min;eigenv;lambda_max]';
 p = poly(lambda); % coefficients of the characteristic polynomial (since R is diagonal)
 p = fliplr(p); 
 
 L = 30;

 w  = zeros(M,1); % initial weight estimate
 u  = zeros(1,M); % initial regressor

 msd_curve_sim = zeros(1,N); % simulated MSD curve     
 mse_curve_sim = zeros(1,N); % simulated learning curve

 %%%%%%%%%%%%%%%%% GENERATE ENSEMBLE-AVERAGE MSD CURVE 

 
 for j=1:L % iteration over learning curves
   j 
   w = zeros(M,1);
   nw = zeros(1,N);
   error = zeros(1,N);
   
   for i=1:N  
     nw(i) = norm(wo-w)^2;
     u = sqrt(12)*(rand(1,M)-0.5)*sqR; % regressor/ uniform distribution with variance R
     d = u*wo + sqn*randn; % reference with Gaussian noise
     error(i) = d - u*w; % error
     if abs(error(i)) >= 1
       w = w + mu*u'*error(i);  % LMS recursion
     else
       w = w + mu*u'*(error(i))^3;  % LMF recursion
     end       
   end
        
   msd_curve_sim = msd_curve_sim + nw;
   mse_curve_sim = mse_curve_sim + abs(error).^2;

end

mse_curve_sim  = mse_curve_sim/L;
msd_curve_sim  = msd_curve_sim/L;
msd_value  = mean(msd_curve_sim(N-200:N));


%%%%%%%%%%%%%%%%% GENERATE THEORETICAL MSD CURVE

 wtilde = wo; % initial condition
 W = zeros(M,1); % state vector
 Y = zeros(M,1);
 Rp1 = eye(M,M);
 Rp2 = R;
 for i=1:M
    Y(i) = trace(Rp2);
    W(i) = wtilde'*Rp1*wtilde;
    Rp2 = R*Rp2;
    Rp1 = R*Rp1;
 end
 
 msd = zeros(1,N);
 mse = zeros(1,N);
 
 for kk=1:N
  msd(kk) = W(1);
  mse(kk) = W(2);

  hg = 3*(var_n + W(2));  % h_g for LMF
  hG = real(hg);
  hU = 15*(W(2)^3) + 45*var_n*(W(2)^2) + 15*xi_v_4*W(2) + xi_v_6; % h_U for LMF
  %hU = 15*(var_n + W(2))^3; % can also be used


  W_next = W;
  W(1:M-1) = W_next(1:M-1) - 2*mu*hG*W_next(2:M);
  W(M) = 2*mu*hG*p(1:M)*W_next(:) + W_next(M);  % computed F*W
  W = W + mu^2*hU*Y;
  
  %F = zeros(M,M);    %Alternative way of updating W
  %F(1,1) = 1;
  %F(1,2) = -2*mu*hG;
 
  %for i=2:M-1
  % F(i,:) = [0 F(i-1,1:M-1)];  
  %end
 
  %F(M,:) = 2*mu*hG*ones(1,M);
  %F(M,:) = F(M,:).*p(1:M);
  %F(M,M) = F(M,M)+1;

  %W = F*W + mu*mu*hU*Y;
 
 end


figure
subplot(211)
plot(1:N,10*log10(msd_curve_sim),'r',1:N,10*log10(msd),'b-')
xlabel('i');
ylabel('MSD (dB)');
legend('Simulation','Theory');
axis tight
grid

subplot(212)
plot(1:N,10*log10(mse_curve_sim),'r',1:N,10*log10(mse),'b-')
xlabel('i');
ylabel('MSE (dB)');
legend('Simulation','Theory');
axis tight
grid

