 
% Adaptive acoustic echo canceller

% Solves part C

% Written by Ali H. Sayed 
% August 2002


close all
clear all       % clears all variables

load room.mat;  % loads the room model
load css.mat;   % loads the CSS data

CC = 20; %20 CSS blocs
loudspeaker=[];
for i=1:CC  %  CSS blocks
    loudspeaker = [loudspeaker css];
end


epsilon = 1e-6;
M = 1024; % length of filter in fullband
B = 32;    % block size of data
K = 2*B; % number of subband filters (2M should be divisible by K)
MM = 2*M/K;  % length of each subband filter 
beta = 0.9;
mu = 0.1; 

len = 2^16; % total length of data chosen as power of 2; should be multiple of B
input = loudspeaker(1:len)';
echo = filter(room,1,input);
reference = echo;
e   = zeros(size(input)); % error sequence for NLMS with power normalization
e2  = zeros(size(input)); % error sequence for standard NLMS
hat_d = zeros(size(reference));  % for NLMS with power normalization
hat_d2 = zeros(size(reference)); % for standard NLMS
 
reg = zeros(1,M);       % regression vector of fullband filter 
wideband = zeros(M,1);  % wideband filter coefficients for NLMS with power normalization
wideband2 = zeros(M,1); % wideband filter coefficients for standard NLMS
 
regressor = zeros(K,MM);  % regression vectors of subband filters 
w = zeros(MM,K);          % coefficients of subband filters for NLMS with power normalization
w2 = zeros(MM,K);         % coefficients of subband filters for standard NLMS
block_u = zeros(K,1); 
D = epsilon*eye(2*B);
  
%% The data is processed in blocks of size B each. 
%% Two adaptation algorithms are run in parallel for comparison purposes:
%% one is NLMS with power normalization and the other is standard NLMS

for i=1:len/B  
   i
   for j=1:B % compute a block of error signals
      reg = [input((i-1)*B+j) reg(1:M-1)];    
      hat_d((i-1)*B+j) = reg*wideband; 
      e((i-1)*B+j) = reference((i-1)*B+j) - hat_d((i-1)*B+j); 
      hat_d2((i-1)*B+j) = reg*wideband2; 
      e2((i-1)*B+j) = reference((i-1)*B+j) - hat_d2((i-1)*B+j); 

   end 
      
   block_u = [flipud(input((i-1)*B+1:i*B)); block_u(1:B)]; % top block is shifted down
   block_e = [flipud(e((i-1)*B+1:i*B)); zeros(B,1)]; % bottom block is always zero 
   block_e2 = [flipud(e2((i-1)*B+1:i*B)); zeros(B,1)]; % bottom block is always zero 

   block_u_fft = (1/sqrt(K))*fft(block_u);   
   block_e_fft = (1/sqrt(K))*fft(block_e);   
   block_e2_fft = (1/sqrt(K))*fft(block_e2);   

   for j=1:K
    D(j) = beta*D(j) + (1-beta)*abs(block_u_fft(j))^2;
   end
   
   
   for j=1:K 
     regressor(j,:) = [block_u_fft(j) regressor(j,1:MM-1)]; 
     w(:,j) = w(:,j) + (mu/MM)*(regressor(j,:)'*block_e_fft(j))/D(j);   % power normalization with step-size scaled by MM
     w2(:,j) = w2(:,j)+(mu*regressor(j,:)'*block_e2_fft(j)/(norm(regressor(j,:))^2+epsilon));  % NLMS 
   end 
    
   g = (1/sqrt(K))*fft(w.');  % The columns of w are transposed without complex conjugation. 
   G = real(g(1:B,:));  % Enforcing the constraint.
   for j = 1:MM 
      wideband((j-1)*B+1:j*B) = G(:,j); 
   end 
    
   g2 = (1/sqrt(K))*fft(w2.');  % The columns of w2 are transposed without complex conjugation. 
   G2 = real(g2(1:B,:));  
   for j = 1:MM 
      wideband2((j-1)*B+1:j*B) = G2(:,j); 
   end 

end

error = e;
error2 = e2;

 subplot(411);
 plot(input); 
 grid;
 title('Loudspeaker');
 axis([0 len -5 5]); 
 subplot(412);
 plot(echo); grid;
 title('Echo');
 axis([0 len -5 5]); 
 subplot(413);
 plot(error); grid;
 title('Error');
 axis([0 len -5 5]); 
 subplot(414);
 plot(error2); grid;
 title('Error by standard NLMS');
 xlabel('i');
 axis([0 len -5 5]); 
 
 mis = 10*log10((norm(wideband-room)^2)/(norm(room)^2)) % relative mismatch by NLMS with power normalization
 mis2 = 10*log10((norm(wideband2-room)^2)/(norm(room)^2)) % relative mismatch by standard NLMS
