
% Adaptive acoustic echo canceller

% Solves part F

% Written by Ali H. Sayed 
% September 2002


close all
clear all       % clears all variables


M = 1024;
load room;

epsilon = 1e-6;
beta = 0.9;


len = 2^16;
L = 50; % 50 experiments

learn_nlms = zeros(len,1);
learn_dft = zeros(len,1);

pole = 0.95; % AR(1) input process
SNR = 30;
var_v = 10^(-SNR/10);

for ll=1:L
 ll    

 loudspeaker = filter(1,[1 -pole],randn(1,len));
 var_l = norm(loudspeaker)^2/len;
 input = loudspeaker(1:len)'/sqrt(var_l);  % unit variance AR(1) process

 v = sqrt(var_v)*randn(len,1);
 echo = filter(room,1,input) + v;
 referetence = echo;

 mu_nlms = 0.5;

 w = zeros(M,1);   
 u = zeros(1,M);
 e = zeros(len,1);
 
 for i=1:len 
    u = [loudspeaker(i) u(1:M-1)];
    e(i) = reference(i) - u*w;
    factor = epsilon + norm(u)^2;
    w = w + mu_nlms*u'*e(i)/factor;  % NLMS recursion
  end

  learn_nlms = learn_nlms + abs(e).^2;
 
  
 %%% Constrained DFT block adaptive implementation

 B = 32;    % block size of data
 K = 2*B; % number of subband filters (2M should be divisible by K)
 MM = M/B;  % length of each subband filter  

 e   = zeros(size(input)); % error sequence for NLMS with power normalization
 hat_d = zeros(size(reference));  % for NLMS with power normalization
 
 reg = zeros(1,M);       % regression vector of fullband filter 
 wideband = zeros(M,1);  % wideband filter coefficients for NLMS with power normalization
 regressor = zeros(K,MM);  % regression vectors of subband filters 
 w = zeros(MM,K);          % coefficients of subband filters for NLMS with power normalization
 block_u = zeros(K,1); 
 D = epsilon*eye(2*B);
 
 mu = 0.1;
 
 %% The data is processed in blocks of size B each. 

 for i=1:len/B  
   for j=1:B % compute a block of error signals
      reg = [input((i-1)*B+j) reg(1:M-1)];    
      hat_d((i-1)*B+j) = reg*wideband; 
      e((i-1)*B+j) = reference((i-1)*B+j) - hat_d((i-1)*B+j); 
   end 
      
   block_u = [flipud(input((i-1)*B+1:i*B)); block_u(1:B)]; % top block is shifted down
   block_e = [flipud(e((i-1)*B+1:i*B)); zeros(B,1)]; % bottom block is always zero 
   block_u_fft = (1/sqrt(K))*fft(block_u);   
   block_e_fft = (1/sqrt(K))*fft(block_e);   

   for j=1:K
    D(j) = beta*D(j) + (1-beta)*abs(block_u_fft(j))^2;
   end
   
   
   for j=1:K 
     regressor(j,:) = [block_u_fft(j) regressor(j,1:MM-1)]; 
     w(:,j) = w(:,j) + (mu/MM)*(regressor(j,:)'*block_e_fft(j))/D(j);   % power normalization with step-size scaled by MM
   end 
    
   g = (1/sqrt(K))*fft(w.');  % The columns of w are transposed without complex conjugation. 
   G = real(g(1:B,:));  % Enforcing the constraint.
   for j = 1:MM 
      wideband((j-1)*B+1:j*B) = G(:,j); 
   end 
 end

 error_dft = e;
 learn_dft = learn_dft + abs(error_dft).^2;

end

learn_dft = 10*log10(learn_dft/L);
learn_nlms = 10*log10(learn_nlms/L);


%smoothing
window = 10;
for i = 1:len-window
  learn_xnlms(i) = mean(learn_nlms(i:i+window));
  learn_xdft(i) = mean(learn_dft(i:i+window));
end


 figure
 plot(1:len,learn_xdft,'b-',1:len,learn_xnlms,'k');
 xlabel('i');
 ylabel('E|e(i)|^2 in dB');
 legend('DFT','NLMS');
 axis tight
 grid
