
% Adaptive acoustic echo canceller

% Solves part F

% Written by Ali H. Sayed 
% September 2002


close all
clear all       % clears all variables

load room.mat;  % loads the room model
load css;   % loads the CSS data

epsilon = 1e-6;
M = 1024; % length of filter in fullband
B = 32;    % block size of data
MM = M/B;  % length of each subband filter 
beta = 0.9;
mu = 0.1; 

K = 3*B;  % number of subband filters for DHT-block implementation



CC = 20; %20 CSS blocs
loudspeaker=[];
for i=1:CC  %  CSS blocks
    loudspeaker = [loudspeaker css];
end

len = 2^16;
input = loudspeaker(1:len)';
echo = filter(room,1,input);
reference = echo;
e   = zeros(size(input)); % error sequence for NLMS with power normalization
hat_d = zeros(size(reference));  % for NLMS with power normalization
 
reg = zeros(1,M);       % regression vector of fullband filter 
wideband = zeros(M,1);  % wideband filter coefficients for NLMS with power normalization
 
regressor = zeros(K,MM);  % regression vectors of subband filters 
w = zeros(MM,K);          % coefficients of subband filters for NLMS with power normalization
block_u = zeros(K,1); 


for m=0:K-1
  for k=0:K-1
    H(m+1,k+1) = 1/sqrt(K)*(cos(2*m*k*pi/K) - sin(2*m*k*pi/K));  % DHT matrix
  end
end

D = epsilon*eye(K);

Ip = zeros(B,B);  % reversed identity
for m=0:B-1
   Ip(m+1,B-m) = 1;
end


%% The data is processed in blocks of size B each. 

for i=1:len/B 
    i
   for j=1:B % compute a block of error signals
      reg = [input((i-1)*B+j) reg(1:M-1)];    
      hat_d((i-1)*B+j) = reg*wideband; 
      e((i-1)*B+j) = reference((i-1)*B+j) - hat_d((i-1)*B+j); 
   end 
      
   block_u = [flipud(input((i-1)*B+1:i*B)); block_u(1:B)]; % top block is shifted down
   block_e = [flipud(e((i-1)*B+1:i*B)); zeros(K-B,1)]; % bottom block is always zero 
   block_u_dht = H*([0; block_u(1:2*B-1); zeros(B,1)]);
   block_e_dht = H*block_e;   

   for j=1:K
     D(j) = beta*D(j) + (1-beta)*abs(block_u_dht(j))^2;
   end
   
   
   for j=1:K 
     regressor(j,:) = [block_u_dht(j) regressor(j,1:MM-1)]; 
     w(:,j) = w(:,j) + (mu/MM)*(regressor(j,:)'*block_e_dht(j))/D(j);   % power normalization with step-size scaled by MM
   end 

   g = H*(w.');  % The columns of w are transposed without complex conjugation. 
   G = [zeros(B,1) eye(B,B) zeros(B,B-1) Ip]*g;  % Enforcing the constraint.
   for j = 1:MM 
     wideband((j-1)*B+1:j*B) = G(:,j); 
   end 
   wideband = (1/2)*wideband;
   
end

 error = e;

 figure
 subplot(311);
 plot(loudspeaker); 
 grid;
 title('Loudspeaker');
 axis([0 len-1 -5 5]); 
 subplot(312);
 plot(echo); grid;
 title('Echo');
 axis([0 len-1 -5 5]); 
 subplot(313);
 plot(error); grid;
 title('Error');
 axis([0 len-1 -5 5]); 
 xlabel('i');
 
