
%CDMA
%part D

%*********************************** CDMA *********************************
%********* Two Users, Rake With Channel Estimation, Orthonal or Non-Orthogonal
%*********                      PN Sequences
%**************************************************************************
clear all;

%Non-Orthogonal PN Sequences
s1=[1,-1,-1,-1,-1,1,1,1,-1,-1,1,1,-1,1,1,-1]; 
s2=[-1,-1,1,-1,1,-1,-1,-1,1,-1,-1,1,-1,1,-1,1];

Lc=16;

%Channel multipath
h=[1 .4;.8 .3];
e1=5;
e2=7;
snr=-20:1:10;

%noise variance for different SNR
Sv=norm(h(1,:))*(10.^(-snr/10));

%Number of Symbols in each SNR For Channel Estimation
N=500;
S1=[];
S2=[];

%******************Tranmitter*************************
   for j=1:length(snr)
   v=(Sv(j))^.5*randn(1,Lc*N+e2);
   b1=ones(N,1);  %user 1 pilots signals
   b2=ones(N,1);  %user 2 pilots signals
   t1=[];
   t2=[];
   for i=1:N
      t1=[t1,b1(i)*s1];
      t2=[t2,b2(i)*s2];
   end
   
   T=[t1 zeros(1,e1+e2-e1)]*h(1,1)+h(1,2)*[zeros(1,e1) t1 zeros(1,e2-e1)]+[t2 zeros(1,e2)]*h(2,1)+h(2,2)*[zeros(1,e2) t2];

%******************Receiver*************************
%Received Data for N symbol
   R=T+v;   
%Correlation
   y1=[];
   y2=[];
   for i=1:N
      y11(i)=h(1,1)*R(Lc*(i-1)+1:Lc*i)*s1';
      y12(i)=h(1,2)*R(Lc*(i-1)+1+e1:Lc*i+e1)*s1';
      y21(i)=h(2,1)*R(Lc*(i-1)+1:Lc*i)*s2';
      y22(i)=h(2,2)*R(Lc*(i-1)+1+e2:Lc*i+e2)*s2';
   end
   %*************************************************** 
   h_hat(1,1,j)=1/Lc*mean(y11);
   h_hat(1,2,j)=1/Lc*mean(y12);
   h_hat(2,1,j)=1/Lc*mean(y21);
   h_hat(1,2,j)=1/Lc*mean(y22);
   %*********Error Probabilitis ***********************
end

%****************************************************************
%****************************************************************
%******   Using Rake Receiver with above Channel Estimation******
%****************************************************************
%****************************************************************
N=200;
S1=[];
S2=[];

%Number of Time Repeating 
Navg=400;
%******************Tranmitter*************************
for t=1:Navg
   ['Progress: ',num2str(t*100/Navg),'%']
   for j=1:length(snr)
   v=(Sv(j))^.5*randn(1,Lc*N+e2);
   b1=2*(randint(N,1)-0.5);  %user 1 data transmitting
   b2=2*(randint(N,1)-0.5);  %user 2 data transmitting
   t1=[];
   t2=[];
   for i=1:N
      t1=[t1,b1(i)*s1];
      t2=[t2,b2(i)*s2];
   end
   T=[t1 zeros(1,e1+e2-e1)]*h(1,1)+h(1,2)*[zeros(1,e1) t1 zeros(1,e2-e1)]+[t2 zeros(1,e2)]*h(2,1)+h(2,2)*[zeros(1,e2) t2];
%******************Receiver*************************
%Received Data for N symbol
   R=T+v;   
%Correlation
   y1=[];
   y2=[];
%******** In this part we will use the estimated Channel  h_hat *****
   for i=1:N
      %*************** Using Estimated Channel *************
      y11_es(i)=h_hat(1,1,j)*R(Lc*(i-1)+1:Lc*i)*s1';
      y12_es(i)=h_hat(1,2,j)*R(Lc*(i-1)+1+e1:Lc*i+e1)*s1';
      y21_es(i)=h_hat(2,1,j)*R(Lc*(i-1)+1:Lc*i)*s2';
      y22_es(i)=h_hat(2,2,j)*R(Lc*(i-1)+1+e2:Lc*i+e2)*s2';
      
      %*************** Using Known Channel *************
      y11(i)=h(1,1)*R(Lc*(i-1)+1:Lc*i)*s1';
      y12(i)=h(1,2)*R(Lc*(i-1)+1+e1:Lc*i+e1)*s1';
      y21(i)=h(2,1)*R(Lc*(i-1)+1:Lc*i)*s2';
      y22(i)=h(2,2)*R(Lc*(i-1)+1+e2:Lc*i+e2)*s2';
  end

   %*************** Using Known Channel *************
   bhat1=sign(y11+y12);
   bhat2=sign(y21+y22);
   %*************** Using Estimated Channel *************
   bhat1_es=sign(y11_es+y12_es);
   bhat2_es=sign(y21_es+y22_es);
  
   %*********Error Probabilitis ***********************
   er1(j)=(length(find(bhat1(1:N)~=b1')))/N;
   er2(j)=(length(find(bhat2(1:N)~=b2')))/N;
   
   er1_es(j)=(length(find(bhat1_es(1:N)~=b1')))/N;
   er2_es(j)=(length(find(bhat2_es(1:N)~=b2')))/N;
   
end
eer1(t,:)=er1;
eer2(t,:)=er2;

eer1_es(t,:)=er1_es;
eer2_es(t,:)=er2_es;

end

%********************Plotting************************
BER1=mean(eer1);
BER2=mean(eer2);

BER1_es=mean(eer1_es);
BER2_es=mean(eer2_es);

semilogy(snr,BER1,'rs-',snr,BER1_es,'bo-');
legend('Using Known Channel','Estimating Channel');
grid on;
xlabel('SNR (dB)');
ylabel('BER');
title('Multipath channel- Two users -Non-Orthogonal PN Sequences');


   

    

   
   
   
   
   
   



 
