
% OFDM receiver
% Part B

clear all
close all

%% Parameters
N = 64; 	% Number of OFDM tones (order of the FFT)
P = 16;     % Cyclic prefix length
M = 8; 	    % Channel length
num_ch = 100;      % Number of channels simulated for BER curves
snr = [0:2:26];	  % Range of SNR simulated for BER curves
num_p = 1; 		  % Number of packets sent
len_p = 16; 		  % Length of each packet sent; i.e., number of symbols
QAM = 2; 		  % Number of constellation points is 2^QAM
p_ind = [4:8:63]; % indices of the tones used as pilot tones
d_ind = setdiff([1:N],p_ind);	% indices of the tones used as data tones
p_sym = 1;
f = fft(eye(N))/sqrt(N);

for ch_cntr=1:num_ch
   ch_cntr

   %h is the complex channel taps
    h = (randn(1,M)+sqrt(-1)*randn(1,M))/sqrt(2*M);
    h_p = [h zeros(1,N-M)];

   %H is the circulant channel matrix
   H = zeros(0,0);
   H = h_p ;

   for c=2:N
      h_p_shift=h_p([N-c+2:N,1:N-c+1]);
      H = [H; h_p_shift];
   end 

   for r=1:length(snr)
     for p=1:num_p
        clear est_s1 est_s2 est_s3;
 	 	bb = round(rand((N-length(p_ind))*len_p,QAM));
        sb = bin2qam(bb,QAM);
        xb(d_ind,:) = reshape(sb,N-length(p_ind),len_p);
        xb(p_ind,:) = (1+sqrt(-1))/sqrt(2)*ones(length(p_ind),len_p);
        x_fb = inv(f)*xb;
        vb =(randn(N+P-M,len_p)+sqrt(-1)*randn(N+P-M,len_p))/sqrt(2);
        yb = H*x_fb+sqrt(10^(-snr(r)/10))*vb(1:N,:);
        y_fb = f*yb;      

  	 	ba = round(rand(N*(len_p-p_sym),QAM));
        sa = bin2qam(ba,QAM);
        xa(1:N,p_sym+1:len_p) = reshape(sa,N,len_p-p_sym);
        xa(1:N,1:p_sym) = (1+sqrt(-1))/sqrt(2)*ones(N,p_sym);
        x_fa = inv(f)*xa;
 	    va = (randn(N+P-M,len_p)+sqrt(-1)*randn(N+P-M,len_p))/sqrt(2);
        ya = H*x_fa+sqrt(10^(-snr(r)/10))*va(1:N,:);
        y_fa = f*ya;      
  
        Qa = inv(f);
   	    Qa = Qa(1:N,1:M);

        %Channel estimate when all tones of the first symbol are used for training
        %Structure (a)

        for i=1:p_sym
         h_hata(:,i) = inv(Qa'*Qa)*Qa'*((y_fa(1:N,i)).*(conj(xa(1:N,i))))/sqrt(N);    
        end

	   h_hata = mean(h_hata,2);

      %Channel estimate when multiple symbols are used for training 
   	  %Structure (b)      

	   Qb = inv(f);
       Qb = Qb(p_ind,1:M);

       for i=1:len_p
	     h_hatb(:,i) = inv(Qb'*Qb)*Qb'*((y_fb(p_ind,i)).*(conj(xb(p_ind,i))))/sqrt(N);
       end

	   h_hatb = mean(h_hatb,2);
       k1 = diag(inv(f)*h_p.');    
       est_s1 = inv(k1)*y_fb/sqrt(N);
       est_s1 = est_s1(d_ind,:);
       est_s1 = est_s1(:);
       b1_est = qam2bin(est_s1,QAM);
       e1_cntr(p) = length(find(bb(:)-b1_est(:)));

       k2 = diag(inv(f)*[h_hatb; zeros(N-M,1)]);    
       est_s2 = inv(k2)*y_fb/sqrt(N);
       est_s2 = est_s2(d_ind,:);
       est_s2 = est_s2(:);
       b2_est = qam2bin(est_s2,QAM);
       e2_cntr(p) = length(find(bb(:)-b2_est(:)));

       k3 = diag(inv(f)*[h_hata; zeros(N-M,1)]);    
       est_s3 = inv(k3)*y_fa(:,p_sym+1:len_p)/sqrt(N);
       est_s3 = est_s3(1:N,:);
       est_s3 = est_s3(:);
       b3_est = qam2bin(est_s3,QAM);
       e3_cntr(p) = length(find(ba(:)-b3_est(:)));
	end

    num_e1(r,ch_cntr) = sum(e1_cntr);
    num_e2(r,ch_cntr) = sum(e2_cntr);
    num_e3(r,ch_cntr) = sum(e3_cntr);
  end
end

bit_e1 = sum(num_e1,2)/(num_ch*num_p*len_p*(N-length(p_ind))*QAM);
bit_e2 = sum(num_e2,2)/(num_ch*num_p*len_p*(N-length(p_ind))*QAM);
bit_e3 = sum(num_e3,2)/(num_ch*num_p*(len_p-p_sym)*(N)*QAM);


figure
semilogy(snr(1:11),bit_e1(1:11),'kv')
hold on
semilogy(snr(1:11),bit_e3(1:11),'bo')
semilogy(snr(1:11),bit_e2(1:11),'r*')
semilogy(snr(1:11),bit_e1(1:11),'k-')
semilogy(snr(1:11),bit_e3(1:11),'b-')
semilogy(snr(1:11),bit_e2(1:11),'r-')
grid on
title(['N=',num2str(N),', P=',num2str(P),', M=',num2str(M),', QAM=',num2str(QAM),', NUM-CH=',num2str(num_ch)]);
xlabel('SNR(dB)');
ylabel('BER');
legend('Exact Channel','Scheme (a)','Scheme (b)');



figure
subplot(2,2,1)
plot(sb,'b+')
grid on
title('transmitted contellation')

subplot(2,2,2)
plot(yb,'b+')
grid on
title('received signal (SNR=25dB)')

subplot(2,2,3)
plot(est_s1,'b+')
grid on
title('using exact channel')

subplot(2,2,4)
plot(est_s2,'b+')
grid on
title('using scheme (a)')

subplot(2,2,1)
axis([-1,1,-1,1]*2)

subplot(2,2,2)
axis([-1,1,-1,1]*2)

subplot(2,2,3)
axis([-1,1,-1,1]*2)

subplot(2,2,4)
axis([-1,1,-1,1]*2)
