% Channel estimation with sufficient excitation
% Solves part A

% Ali H. Sayed
% Sep. 2002.

close all  % closes all open figures
clear all  % clears all variables

N = 500;
M = 32;
channel = randn(M,1);
channel = channel/norm(channel);

SNR = 30;
var_v = 10^(-SNR/10);

%load channel


epsilon = 1e-6;
      
lambda_vec = [0.85 0.90 0.91 0.92 0.93 0.94 0.95 0.96 0.97 0.98 0.985 0.99 0.995 1];


K = length(lambda_vec);
S = zeros(K,1);
condP = zeros(K,1);

input = randn(1,N);

for ii=1:K % iteration over forgetting factors
  w  = zeros(M,1); % initial weight estimate
  u  = zeros(1,M); % initial regressor
  lambda = lambda_vec(ii);  
  [ii lambda]
  P = (1/epsilon)*eye(M,M);
  for i=1:N  
    u = [input(i) u(1:M-1)]; % regressor
    d = u*channel + sqrt(var_v)*randn; % reference
    error = d - u*w; % error            
    gamma = 1/(1 +(1/lambda)*u*P*u'); % conversion factor
    g = (1/lambda)*P*u'*gamma; % gain vevtor
    P = (1/lambda)*P - (g*g'/gamma); % Riccati variable
    w = w + P*u'*error;  % RLS recursion
  end
  S(ii) = (norm(w-channel)^2)/(norm(channel)^2);
  condP(ii) = cond(P);
end

subplot(221)
plot(lambda_vec,10*log10(S),'b-d');
xlabel('\lambda')
ylabel('mismatch(dB)')
axis tight
grid

subplot(222)
plot(lambda_vec,20*log10(condP),'b-d');
xlabel('\lambda')
ylabel('\kappa(P) in dB scale')
axis tight
grid

subplot(224)
plot(lambda_vec(K-4:K),condP(K-4:K),'b-d');
xlabel('\lambda')
ylabel('\kappa(P) in linear scale')
axis tight
grid
