% Channel estimation with sufficient excitation
% Solves part D

% Ali H. Sayed
% Sep. 2002.

close all  % closes all open figures
clear all  % clears all variables

N = 2500;
M = 32;
channel = randn(M,1);
channel = channel/norm(channel);

SNR = 30;
var_v = 10^(-SNR/10);

epsilon = 1e-6;
      
mu_vec = [0.01 0.05 0.1 0.3 0.5 0.7 1];

K = length(mu_vec);
S = zeros(K,1);

input = randn(1,N);

for ii=1:K % iteration over forgetting factors
  w  = zeros(M,1); % initial weight estimate
  u  = zeros(1,M); % initial regressor
  mu = mu_vec(ii);  
  [ii mu]
  for i=1:N  
    u = [input(i) u(1:M-1)]; % regressor
    d = u*channel + sqrt(var_v)*randn; % reference
    error = d - u*w; % error            
    factor = epsilon + norm(u)^2;
    w = w + mu*u'*error/factor;  % NLMS recursion
  end
  S(ii) = (norm(w-channel)^2)/(norm(channel)^2);
end

subplot(321)
plot(mu_vec,10*log10(S),'b-d');
xlabel('\mu')
ylabel('mismatch(dB)')
axis tight
grid

subplot(322)
plot(input(1:50));
xlabel('i')
axis tight
grid


%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%% Sinusoidal input
%%%%%%%%%%%%%%%%%%%%%%%%%%%

mu_vec = 1e-3*[0.01 0.05 0.1 0.3 0.5 0.7 1];

K = length(mu_vec);
S1 = zeros(K,1);

fo = 1240; %Hz
input = sin(2*pi*fo/8000*[0:N-1]); % sampling assumed at 8KHz
power = norm(input)^2/N;
input = input/sqrt(power);

for ii=1:K % iteration over forgetting factors
  w  = zeros(M,1); % initial weight estimate
  u  = zeros(1,M); % initial regressor
  mu = mu_vec(ii);  
  [ii mu]
  for i=1:N  
    u = [input(i) u(1:M-1)]; % regressor
    d = u*channel + sqrt(var_v)*randn; % reference
    error = d - u*w; % error            
    factor = epsilon + norm(u)^2;
    w = w + mu*u'*error/factor;  % NLMS recursion
  end
  S1(ii) = (norm(w-channel)^2)/(norm(channel)^2);
end

subplot(323)
plot(mu_vec,10*log10(S1),'b-d');
xlabel('\mu')
ylabel('mismatch(dB)')
axis tight
grid

subplot(324)
plot(input(1:50));
xlabel('i')
axis tight
grid


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%% Multiple sinusoidal input
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

mu_vec = [0.01 0.05 0.1 0.3 0.5 0.7 1];

K = length(mu_vec);
S2 = zeros(K,1);

input = zeros(1,N);
fo = 1240;  %Hz
Lsin = 100;  %number of sinusoids
for i=1:Lsin
  input = input + sin((2*pi*i*fo/(8000*Lsin))*[0:N-1]);
end
power = norm(input)^2/N;
input = input/sqrt(power);

for ii=1:K % iteration over forgetting factors
  w  = zeros(M,1); % initial weight estimate
  u  = zeros(1,M); % initial regressor
  mu = mu_vec(ii);  
  [ii mu]
  for i=1:N  
    u = [input(i) u(1:M-1)]; % regressor
    d = u*channel + sqrt(var_v)*randn; % reference
    error = d - u*w; % error         
    factor = epsilon + norm(u)^2;
    w = w + mu*u'*error/factor;  % NLMS recursion
  end
  S2(ii) = (norm(w-channel)^2)/(norm(channel)^2);
end

subplot(325)
plot(mu_vec,10*log10(S2),'b-d');
xlabel('\mu')
ylabel('mismatch(dB)')
axis tight
grid

subplot(326)
plot(input(1:50));
xlabel('i')
axis tight
grid

