
% Quantization effects and array algorithms
% Solves part B

% A. H. Sayed
% Sep. 2002.

close all
clear all

var_n = 1e-3;
sqn = sqrt(var_n);

N = 200;

epsilon = 1e-6;
lambda = 0.995;

M = 5;
wo  = randn(M,1);
wo = wo/norm(wo);

learn_rls = zeros(1,N);
learn_array = zeros(1,N);

L = 100;

Br = 30; % bits for data (including sign bit)
Bc = 30; % bits for coefficients (including sign bit)

for k = 1:L
 k   
 
 input = randn(1,N);
 v = sqn*randn(1,N); 

 e_rls = zeros(1,N);
 e_array = zeros(1,N);

 u = zeros(1,M);

 w_rls = zeros(M,1);
 
 P_rls = (1/epsilon)*eye(M,M);
 Phi = epsilon^(1/2)*eye(M,M);
 q = zeros(M,1);
 
 
   for j=1:N
     u = [input(j) u(1:M-1)];
     d = u*wo + v(j);
            
     e_rls(j) = d - quantize(u*w_rls,Br); 
     e_rls(j) = quantize(e_rls(j),Br);
     
     aa = quantize_v(P_rls*u',Br);
     bb = quantize((1/lambda)*u*aa,Br);
     gamma = 1/(1+bb); 
     gamma = quantize(gamma,Br);
     
     g = quantize_v((1/lambda)*aa*gamma,Bc); 
     w_rls = w_rls + quantize_v(g*e_rls(j),Bc);  % quantized version of RLS 
     w_rls = quantize_v(w_rls,Bc);
     
     P_rls = quantize_v((1/lambda)*P_rls,Br) - quantize_v((g*g'/gamma),Br);
     P_rls = quantize_v(P_rls,Br);
 

     A = [quantize_v(lambda^(1/2)*Phi,Br)  u'; quantize_v(lambda^(1/2)*q',Br) d'; zeros(1,M) 1];
     [c,s] = givens(A(1,1),A(1,M+1));  
     c = quantize(c,Br);
     s = quantize(s,Br);
     
     B = A*[c zeros(1,M-1) s; zeros(M-1,1) eye(M-1,M-1) zeros(M-1,1);conj(s) zeros(1,M-1) -c]; 
     B = quantize_v(B,Br);
     
     for kk=2:M
      [c,s] = givens(B(kk,kk),B(kk,M+1));  
      c = quantize(c,Br);
      s = quantize(s,Br);
      
      B = B*[eye(kk-1,kk-1) zeros(kk-1,M-kk+2); zeros(1,kk-1) c zeros(1,M-kk) s; zeros(M-kk,kk) eye(M-kk,M-kk) zeros(M-kk,1);zeros(1,kk-1) conj(s) zeros(1,M-kk) -c];
      B = quantize_v(B,Br);
 
     end
      
     Phi = B(1:M,1:M);
     q = (B(M+1,1:M))';
     e_array(j) = quantize((B(M+1,M+1))'/B(M+2,M+1),Br);

   end
    learn_rls = learn_rls + abs(e_rls).^2;
    learn_array = learn_array + abs(e_array).^2;

end

learn_array = 10*log10(learn_array/L);
learn_rls = 10*log10(learn_rls/L);

figure
subplot(221)
plot(learn_rls)
xlabel('i')
ylabel('dB')
axis tight
grid
legend('RLS')
subplot(222)
plot(learn_array)
grid
xlabel('i')
ylabel('dB')
legend('QR')
axis tight

[Bc Br]