
% Adaptive channel estimation

% Solves part A

% Written by Ali H. Sayed 
% September 2002


close all
clear all       % clears all variables


M = 64;
load channel 
N = 1000;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%% Standard FTF implementation
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

L = 100; % 100 experiments
learn_ftf = zeros(N,1);
SNR = 30;
var_v = 10^(-SNR/10);

for ll=1:L
 ll    

 input = randn(1,N);
 noise = sqrt(var_v)*randn(1,N);

 w = zeros(M,1);   
 u = zeros(1,M);
 e = zeros(N,1);

 wf = zeros(M,1); % forward prediction vector wf
 wb = zeros(M,1); % backward prediction vector wb
 w  = zeros(M,1); % channel taps
 g  = zeros(M,1); % gain vector
 reg = zeros(1,M); % regressor

 lambda = 1- 2^(-10); % forgetting factor -- usually in the range (1-1/3M,1-1/10M)
 inv_delta = 2^(-11); % inverse of regularization factor
 gamma   = 1; % conversion factor
 xif = inv_delta/lambda; % forward prediction energy
 xib = inv_delta/lambda^(M+1); % backward prediction energy

 for i=1:N 
    fa = input(i) - u*wf;      % a-priori forward error
    f  = gamma*fa;              % a-posteriori forward error
    xifo = xif;                   % save forward energy
    xif = lambda*xif + f*fa';    % time-update of forward energy
    gamma1 = gamma*lambda*xifo/xif; % order- and time-update of conversion factor
    g1 = [0;g] + fa'/(lambda*xifo)*[1;-wf]; % order- and time-update of gain vector
     v = g1(M+1);  % last entry of gain vector
    wf = wf + f*g; % time-update of forward prediction taps
    ba = lambda*xib*v;  % a-priori backward error

    if (1-ba*gamma1*v' <= 0)
      ind = num2str(i); 
      str = ['rescue needed but not used at iteration ',ind]; 
      disp(str) % message
    end

    
    gamma = gamma1/(1-ba*gamma1*v'); % order downdate of conversion factor
    g = g1(1:M) + v*wb;          % order-downdate of gain vector 
    b = gamma*ba;                % a-posteriori backward error
    xib = lambda*xib + b*ba';      % time-update of backward energy
    wb = wb + b*g;                % time-update of backward prediction taps
    u = [input(i) u(1:M-1)];
    reference(i) = u*channel + noise(i);
    hat_d = u*w;
    e(i) = reference(i) - hat_d; 
    ep = e(i)*gamma;   % a-posteriori estimation error
    w = w + ep*g;      % time-update of weight vector
    
    
 end  
 learn_ftf = learn_ftf + abs(e).^2;
end

learn_ftf = 10*log10(learn_ftf/L);



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%% FTF implementation with rescue mechanism
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


L = 100; % 100 experiments
learn_ftf_r = zeros(N,1);
SNR = 30;
var_v = 10^(-SNR/10);


for ll=1:L
 ll    

 input = randn(1,N);
 noise = sqrt(var_v)*randn(1,N);

 w = zeros(M,1);   
 u = zeros(1,M);
 e = zeros(N,1);

 wf = zeros(M,1); % forward prediction vector wf
 wb = zeros(M,1); % backward prediction vector wb
 w  = zeros(M,1); % channel taps
 g  = zeros(M,1); % gain vector
 reg = zeros(1,M); % regressor

 lambda = 1- 2^(-10); % forgetting factor -- usually in the range (1-1/3M,1-1/10M)
 inv_delta = 2^(-11); % inverse of regularization factor
 gamma   = 1; % conversion factor
 xif = inv_delta/lambda; % forward prediction energy
 xib = inv_delta/lambda^(M+1); % backward prediction energy

for i=1:N 
    fa = input(i) - u*wf;      % a-priori forward error
    f  = gamma*fa;              % a-posteriori forward error
    xifo = xif;                   % save forward energy
    xif = lambda*xif + f*fa';    % time-update of forward energy
    gamma1 = gamma*lambda*xifo/xif; % order- and time-update of conversion factor
    g1 = [0;g] + fa'/(lambda*xifo)*[1;-wf]; % order- and time-update of gain vector
     v = g1(M+1);  % last entry of gain vector
    wf = wf + f*g; % time-update of forward prediction taps
    ba = lambda*xib*v;  % a-priori backward error

    if (1-ba*gamma1*v' > 0)
      gamma = gamma1/(1-ba*gamma1*v'); % order downdate of conversion factor
      g = g1(1:M) + v*wb;          % order-downdate of gain vector 
      b = gamma*ba;                % a-posteriori backward error
      xib = lambda*xib + b*ba';      % time-update of backward energy
      wb = wb + b*g;                % time-update of backward prediction taps
     else % rescue procedure
       ind = num2str(i); 
       str = ['rescue procedure used at iteration ',ind]; 
       disp(str) % message
       gamma = 1;
       g  = zeros(M,1);
       wb = g;
       wf = g;
       b  = ba;
       xif = inv_delta/lambda;
       xib = inv_delta/lambda^(M+1);
     end

     u = [input(i) u(1:M-1)];
     reference(i) = u*channel + noise(i);
     hat_d = u*w;
     e(i) = reference(i) - hat_d; 
     ep = e(i)*gamma;   % a-posteriori estimation error
     w = w + ep*g;      % time-update of weight vector
  end
  learn_ftf_r = learn_ftf_r + abs(e).^2;
end

learn_ftf_r = 10*log10(learn_ftf_r/L);


%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%% Stabilized FTF
%%%%%%%%%%%%%%%%%%%%%%%%%%

L = 100; % 100 experiments
learn_sftf = zeros(N,1);
SNR = 30;
var_v = 10^(-SNR/10);

for ll=1:L
 ll    

 input = randn(1,N);
 noise = sqrt(var_v)*randn(1,N);

 w = zeros(M,1);   
 u = zeros(1,M);
 e = zeros(N,1);

 wf = zeros(M,1); % forward prediction vector wf
 wb = zeros(M,1); % backward prediction vector wb
 w  = zeros(M,1); % channel taps
 g  = zeros(M,1); % gain vector
 reg = zeros(1,M); % regressor

 lambda = 1- 2^(-10); % forgetting factor -- usually in the range (1-1/3M,1-1/10M)
 inv_delta = 2^(-11); % inverse of regularization factor
 gaminv   = 1; % conversion factor
 xifinv = lambda/inv_delta; % forward prediction energy
 xib = inv_delta/lambda^(M+1); % backward prediction energy

 k = [1.5 2.5 1]; % gains for convex combinations 

 for i=1:N 
    
   fa = input(i) - u*wf;  % a-priori forward error
   f  = fa/gaminv;  % a-posteriori forward error 
   g1 = [0;g] + fa'*xifinv/lambda*[1;-wf]; % order- and time-update of gain vector
   v1 = g1(M+1); 
   v2 = g1(1); 
   gaminva = gaminv + v2*fa'; 
   xifinv  = xifinv/lambda - abs(v2)^2/gaminva; 
   wf = wf + f*g; 
   ba1 = lambda*xib*v1;
   ba2 = u(M) - [input(i) u(1:M-1)]*wb; 
   for ii=1:3 
    ba3(ii) = ba2*k(ii) + ba1*(1-k(ii)); 
   end 
   gaminv1 = gaminva - v1*ba3(3)';  
 
   if gaminv1 > 0 
     for ii=1:2 
       b(ii) = ba3(ii)/gaminv1; 
     end 
     xib = lambda*xib + b(2)*ba3(2)';         
     g = g1(1:M) + v1*wb; 
     wb = wb + b(1)*g; 
     gaminv = 1/(lambda^(M)*xifinv*xib); 
   else % rescue procedure 
     ind = num2str(i); 
     str = ['rescue procedure used at iteration ',ind]; 
     disp(str) % message
       
     gaminv = 1;
     g  = zeros(M,1);
     wb = g;
     wf = g;
     xif = lambda/inv_delta; 
     xib = inv_delta/lambda^(M+1);
   end 
 
  u = [input(i) u(1:M-1)]; 
  reference(i) = u*channel + noise(i);
  hat_d = u*w; 
  e(i) = reference(i) - hat_d; 
  ep = e(i)/gaminv; % a-posteriori estimation error
  w = w + ep*g;      % time-update of weight vector
 end    
 learn_sftf = learn_sftf + abs(e).^2;
end

learn_sftf = 10*log10(learn_sftf/L);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%% Fast Array Algorithm
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

Le = 100; % 100 experiments
learn_array = zeros(N,1);
SNR = 30;
var_v = 10^(-SNR/10);

for ll=1:Le
 ll    

 input = randn(1,N);
 noise = sqrt(var_v)*randn(1,N);

 w = zeros(M,1);   
 u = zeros(1,M);
 e = zeros(N,1);

 lambda = 1- 2^(-10);
 eta = 2^(11); 
 
 L = zeros(M+1,2); 
 L(1,1) = sqrt(eta*lambda);
 L(M+1,2)=sqrt(eta*lambda)*lambda^(M/2);
 
 g  = zeros(M,1);  % gain vector
 gammainvsq   = 1; % square-root of inverse conversion factor
 k = g*gammainvsq; % normalized gain

 A = zeros(M+2,3); % pre-array
 B = zeros(M+2,3); % post-array
 
 ux = zeros(1,M+1); % extended regressor

 for i=1:N
   ux = [input(i) u] ;
   A = [gammainvsq ux*L; [0;k] L];

   [c,s] = givens(A(1,1),A(1,2));  % determine Givens rotation c and s parameters
   B = A*[c s 0;conj(s) -c 0;0 0 1]; % apply Givens rotation
   [c,s] = hyperbolic(B(1,1),B(1,3)); % determine hyperbolic rotation parameters

   %[c,s] = hyperbolic(B(M+2,1),B(M+2,3)); % determine hyperbolic rotation parameters  (second version)

   B = B*[c 0 -s;0 1 0;-conj(s) 0 c]; % apply hyperbolic rotation

   L = B(2:M+2,2:3)/sqrt(lambda);
   gammainvsq = B(1,1);
   k = B(2:M+1,1);  
   
   u = [input(i) u(1:M-1)];
   reference(i) = u*channel + noise(i);
   e(i) = reference(i)- u*w;
   w = w + (k/gammainvsq)*e(i);    
 end

 learn_array = learn_array + abs(e).^2;
end

learn_array = 10*log10(learn_array/Le);


figure
subplot(221)
plot(1:N,learn_ftf,'b-');
ylabel('E|e(i)|^2 in dB');
title('FTF');
axis tight
grid

subplot(222)
plot(1:N,learn_ftf_r,'b-');
title('FTF with rescue');
axis tight
grid

subplot(223)
plot(1:N,learn_sftf,'b-');
ylabel('E|e(i)|^2 in dB');
xlabel('i');
title('Stabilized FTF');
axis tight
grid

subplot(224)
plot(1:N,learn_array,'b-');
title('Fast array');
xlabel('i');
axis tight
grid

