
function eo = lat1(input,d,lambda,eta,order);

% A posteriori-based RLS lattice filter
% returns the a-priori output error sequence

% Ali H. Sayed
% Sep. 2002

N = max(size(input));
M = order;

% Initialization of M-order vectors
gamma = ones(M+1,1);
b = zeros(M,1);
delta = zeros(M,1);
rho = zeros(M,1);
xif = (1/eta*lambda*lambda)*ones(M,1);
xib = (1/eta*lambda*lambda)*ones(M,1);
for m=0:M-1
  xib(m+1) = xib(m+1)/lambda^m;
end
      
for i=1:N
  gammao = gamma;          
  bo = b;
  xibo = xib;
  gamma = ones(M,1);
  b(1) = input(i);
  f(1) = input(i);
  r(1) = d(i);
  for m=1:M
     xif(m) = lambda*xif(m) + (abs(f(m)))^2/gammao(m);
     xib(m) = lambda*xib(m) + (abs(b(m)))^2/gamma(m);
                    
     delta(m) = lambda*delta(m) + (f(m))'*(bo(m))/gammao(m);
     rho(m) = lambda*rho(m) + (r(m))'*(b(m))/gamma(m);
      
     gamma(m+1) = gamma(m) - abs(b(m))^2/xib(m);
     kappab(m) = delta(m)/xif(m);
     kappaf(m) = (delta(m))'/xibo(m);
     kappa(m) = (rho(m))'/xib(m);
        
     b(m+1) = bo(m) - kappab(m)*f(m);
     f(m+1) = f(m) - kappaf(m)*bo(m);
     r(m+1) = r(m) - kappa(m)*b(m);
  end
  eo(i) = r(M+1)/gamma(M+1);
 end

