
function eo = lat3(input,d,lambda,eta,order);

% A priori-based error feedback RLS lattice filter
% returns the a-priori output error sequence

% Ali H. Sayed
% Sep. 2002

N = max(size(input));
M = order;

% Initialization of M-order vectors
 gamma = ones(M,1);
 beta = zeros(M,1);
 kappaf = zeros(M,1);
 kappab = zeros(M,1);
 kappa = zeros(M,1);
   
 xif = (1/eta*lambda*lambda)*ones(M,1);
 xib = (1/eta*lambda*lambda)*ones(M,1);
 for m=0:M-1
   xib(m+1) = xib(m+1)/lambda^m;
 end
      
    
 for i=1:N
   gammao = gamma;          
   betao = beta;
   xibo = xib;
   gamma = ones(M+1,1);
   beta(1) = input(i);
   alpha(1) = input(i);
   e(1) = d(i);
   for m=1:M
     xif(m) = lambda*xif(m) + (abs(alpha(m)))^2*gammao(m);
     xib(m) = lambda*xib(m) + (abs(beta(m)))^2*gamma(m);

     beta(m+1) = betao(m) - kappab(m)*alpha(m);
     alpha(m+1) = alpha(m) - kappaf(m)*betao(m);
     e(m+1) = e(m) - kappa(m)*beta(m);

     kappa(m) = kappa(m) + (beta(m))'*gamma(m)*e(m+1)/xib(m);
     kappaf(m) = kappaf(m) + (betao(m))'*gammao(m)*alpha(m+1)/xibo(m);
     kappab(m) = kappab(m) + (alpha(m))'*gammao(m)*beta(m+1)/xif(m);
        
     gamma(m+1) = gamma(m) - abs(gamma(m)*beta(m))^2/xib(m);
   end
   eo(i) = e(M+1);
 end
