
% Laguerre and FIR adaptive filtering
% Solves part A

% Ali H. Sayed
% Sep. 2002

clear all
close all

N = 2000;
SNR = 30;
var_v = 10^(-SNR/10);

num  = 0.00692*[0 1 0.673];
den1 = [1 -0.368];
den2 = [1 -0.819];
den3 = [1 -0.995];

imp = zeros(N,1);
imp(1)=1;
h1 = filter(num,den1,imp);
h2 = filter(1,den2,h1);
h  = filter(1,den3,h2);  % first N samples of impulse response

epsilon = 1e-6;
lambda = 0.999;

Mfir = 700; % filter oder
Mlag = 20;

a = 0.95; %Laguerre pole
sqa = sqrt(1-abs(a)^2);

learn_fir = zeros(1,N);
learn_lag = zeros(1,N);

L = 100;

for k = 1:L
 k   
 
 input = randn(N,1);
 d1 = filter(num,den1,input);
 d2 = filter(1,den2,d1);
 d  = filter(1,den3,d2);
 d = d + sqrt(var_v)*randn(N,1);

 e_fir = zeros(1,N);
 e_lag = zeros(1,N);

 u_fir = zeros(1,Mfir);
 u_lag = zeros(1,Mlag);

 w_fir = zeros(Mfir,1);
 w_lag = zeros(Mlag,1); 

 P_fir = (1/epsilon)*eye(Mfir,Mfir);
 P_lag = (1/epsilon)*eye(Mlag,Mlag);

 
 wf = zeros(Mfir,1);  % I am using fast RLS to avoid a slow simulation
 wb = zeros(Mfir,1);  % since FIR filter is long
 w  = zeros(Mfir,1); 
 g  = zeros(Mfir,1); 
 reg = zeros(1,Mfir); 

 inv_delta = 2^(-11); 
 gamma   = 1; 
 xif = inv_delta/lambda; 
 xib = inv_delta/lambda^(Mfir+1); 


 
 for j=1:N
     
%u_fir = [input(j) u_fir(1:Mfir-1)];
%e_fir(j) = d(j) - u_fir*w_fir; 
%gamma = 1/(1+(1/lambda)*u_fir*P_fir*u_fir'); 
%g = (1/lambda)*P_fir*u_fir'*gamma; 
%w_fir = w_fir + g*e_fir(j);               % FIR RLS (slow simulation)
%P_fir = (1/lambda)*P_fir - (g*g'/gamma);
 

   %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
   %%%%%%%%%%%%%%%%%%%%%%%% FTF instead of RLS to speed up the simulation %%%%%%%%%%%%%%%
   %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    fa = input(j) - u_fir*wf;   % FTF recursions
    f  = gamma*fa;              
    xifo = xif;                 
    xif = lambda*xif + f*fa';   
    gamma1 = gamma*lambda*xifo/xif; 
    g1 = [0;g] + fa'/(lambda*xifo)*[1;-wf]; 
     v = g1(Mfir+1);  
    wf = wf + f*g; 
    ba = lambda*xib*v;  
    gamma = gamma1/(1-ba*gamma1*v'); 
    g = g1(1:Mfir) + v*wb;          
    b = gamma*ba;               
    xib = lambda*xib + b*ba';   
    wb = wb + b*g;              
    u_fir = [input(j) u_fir(1:Mfir-1)];
    hat_d = u_fir*w_fir;
    e_fir(j) = d(j) - hat_d; 
    ep = e_fir(j)*gamma;   
    w_fir = w_fir + ep*g;
    
   %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
   %%%%%%%%%%%%%%%%%%%%%% Laguerre RLS %%%%%%%%%%%%%%%%%%%%%%%%%
   %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

   u_lag_p = u_lag;    
   u_lag(1) = a*u_lag(1) + sqa*input(j);  % output of L_o(z)
   for s = 2:Mlag
    u_lag(s) = a*u_lag(s) - a'*u_lag(s-1) + u_lag_p(s-1);
   end
   
   e_lag(j) = d(j) - u_lag*w_lag; 
   gamma_lag = 1/(1+(1/lambda)*u_lag*P_lag*u_lag'); 
   g_lag = (1/lambda)*P_lag*u_lag'*gamma_lag; 
   w_lag = w_lag + g_lag*e_lag(j);  % Laguerre RLS
   P_lag = (1/lambda)*P_lag - (g_lag*(g_lag)'/gamma_lag);

      
  end
   learn_fir = learn_fir + abs(e_fir).^2;
   learn_lag = learn_lag + abs(e_lag).^2;
end

learn_fir = 10*log10(learn_fir/L);
learn_lag = 10*log10(learn_lag/L);



u_lag = zeros(1,Mlag);
input2 = zeros(1,N);
input2(1) =1;  % Let us now determine the estimated impulse response 

for j=1:N
   u_lag_p = u_lag;    
   u_lag(1) = a*u_lag(1) + sqa*input2(j);  % output of L_o(z)
   for s = 2:Mlag
    u_lag(s) = a*u_lag(s) - a'*u_lag(s-1) + u_lag_p(s-1);
   end
   
   h_hat(j) = u_lag*w_lag; 
end


figure
plot(1:N,learn_fir,'r',1:N,learn_lag,'b')
xlabel('i')
ylabel('dB')
axis tight
grid
legend('FIR','Laguerre')

figure
subplot(211)
plot(1:N,h,'b',1:N,h_hat,'r')
xlabel('i')
legend('actual','estimate by Laguerre')
title('impulse response')
subplot(212)
plot(1:N,[w_fir;zeros(N-Mfir,1)],'b')
xlabel('i')
title('FIR estimated impulse response')


figure
subplot(211)
plot(input)
title('input');
xlabel('i');

subplot(212)
plot(d)
title('reference');
xlabel('i');
