
% Active noise control
% Solves part B

% Ali H. Sayed
% Sep. 2002

clear all
close all

N = 30000;

load path;
M = max(size(path));

SNR = 60;
var_v = 10^(-SNR/10);
F = [1 -1.2 0.72];

alpha_vec = [0.01 0.02 0.03 0.05 0.1];
KK = max(size(alpha_vec));

epsilon = 1e-6;

L = 50; 

learn_alpha = zeros(KK,N);

for k=1:KK
  alpha = alpha_vec(k);
  learn = zeros(1,N);
  [k alpha]
  
  for j=1:L

    u = zeros(1,M);
    w = zeros(M,1);
    e = zeros(1,N);
    e1 = 0;
    e2 = 0;

    input = binary(N);
    %input = randn(1,N);
    d = filter(path,1,input) + sqrt(var_v)*randn(1,N);

    learn(1) = learn(1) + norm(w-path)^2;
    u = [input(1) u(1:M-1)];  % first iteration
    factor = (epsilon+norm(u)^2);
    e(1) = d(1) - u*w;
    ef = F(1)*e(1) + F(2)*e1 + F(3)*e2;
    w = w + (alpha/factor)*u'*ef; 
    

    for i=2:N
      learn(i) = learn(i) + norm(w-path)^2;

      u = [input(i) u(1:M-1)];
      factor = (epsilon+norm(u)^2);
      e2 = e1;  % e(i-2)
      e1 = e(i-1);   % e(i-1)
      e(i) = d(i) - u*w;
      ef = F(1)*e(i) + F(2)*e1 + F(3)*e2;  % filtered error
      w = w + (alpha/factor)*u'*ef;

    end
  end  
  learn_alpha(k,:) = 10*log10(learn/L);  
end


figure
plot(1:N,learn_alpha(1,:),'r',1:N,learn_alpha(2,:),'b',1:N,learn_alpha(3,:),'m',1:N,learn_alpha(4,:),'k',1:N,learn_alpha(5,:),'c');
legend('0.01','0.02','0.03','0.05','0.1');
axis tight
grid
ylabel('dB')
xlabel('i')
