
% Active noise control
% Solves part C

% Ali H. Sayed
% Sep. 2002

clear all
close all

N = 5000;
load path;
M = max(size(path));

SNR = 60;
var_v = 10^(-SNR/10);

F = [1 -1.2 0.72];

alpha_vec = [0.05 0.1 0.2];

KK = max(size(alpha_vec));

epsilon = 1e-6;

L = 50; 

learn_alpha = zeros(KK,N);

for k=1:KK
  alpha = alpha_vec(k);
  learn = zeros(1,N);
  [k alpha]

  for j=1:L

    u = zeros(1,M);
    u1 = zeros(1,M);
    u2 = zeros(1,M);
    w = zeros(M,1);
    e = zeros(1,N);
    e1 = 0;
    e2 = 0;

    input = randn(1,N);
    d = filter(path,1,input) + sqrt(var_v)*randn(1,N);

    u2 = u1; % first iteration
    u1 = u;
    u = [input(1) u(1:M-1)];
    factor = (epsilon+norm(u)^2);
    e(1) = d(1) - u*w;
    ef = F(1)*e(1) + F(2)*e1 + F(3)*e2;
    uf  = F(1)*u + F(2)*u1 + F(3)*u2;  % filtered regressor
    w = w + (alpha/factor)*uf'*ef; 

    for i=2:N
      u2 = u1;
      u1 = u;
      u = [input(i) u(1:M-1)];
      factor = (epsilon+norm(u)^2);
      e2 = e1;  % e(i-2)
      e1 = e(i-1);   % e(i-1)
      e(i) = d(i) - u*w;
      ef = F(1)*e(i) + F(2)*e1 + F(3)*e2;  % filtered error
      uf  = F(1)*u + F(2)*u1 + F(3)*u2;  % filtered regressor
      w = w + (alpha/factor)*uf'*ef;
    end
    learn = learn + abs(e).^2;
  end  
  learn_alpha(k,:) = 10*log10(learn/L);  
end

figure
plot(1:N,learn_alpha(1,:),'r',1:N,learn_alpha(2,:),'b',1:N,learn_alpha(3,:),'m');
legend('0.05','0.1','0.2');
axis tight
grid
ylabel('dB')
xlabel('i')
